Package: RNAdecay
Date: 2023-10-25
Title: Maximum Likelihood Decay Modeling of RNA Degradation Data
Version: 1.33.0
Authors@R: c(person("Reed", "Sorenson", email = "reedssorenson@gmail.com", role = c("aut", "cre"),comment=c(ORCID="0000-0001-8650-0601")), person("Katrina", "Johnson", email = "kjohnson@math.utah.edu", role = c("aut")),person("Frederick", "Adler", email = "adler@math.utah.edu", role = c("aut")),person("Leslie", "Sieburth", email = "sieburth@biology.utah.edu", role = c("aut"),comment=c(ORCID="0000-0001-6691-2000")))
Description: RNA degradation is monitored through measurement of RNA abundance after inhibiting RNA synthesis. This package has functions and example scripts to facilitate (1) data normalization, (2) data modeling using constant decay rate or time-dependent decay rate models, (3) the evaluation of treatment or genotype effects, and (4) plotting of the data and models. Data Normalization:  functions and scripts make easy the normalization to the initial (T0) RNA abundance, as well as a method to correct for artificial inflation of Reads per Million (RPM) abundance in global assessments as the total size of the RNA pool decreases. Modeling: Normalized data is then modeled using maximum likelihood to fit parameters. For making treatment or genotype comparisons (up to four), the modeling step models all possible treatment effects on each gene by repeating the modeling with constraints on the model parameters (i.e., the decay rate of treatments A and B are modeled once with them being equal and again allowing them to both vary independently). Model Selection: The AICc value is calculated for each model, and the model with the lowest AICc is chosen. Modeling results of selected models are then compiled into a single data frame. Graphical Plotting: functions are provided to easily visualize decay data model, or half-life distributions using ggplot2 package functions.
Depends: R (>= 4.3)
Imports: stats, grDevices, grid, ggplot2, gplots, utils, TMB, nloptr,
        scales
Suggests: parallel, knitr, reshape2, rmarkdown
biocViews: ImmunoOncology, Software, GeneExpression, GeneRegulation,
        DifferentialExpression, Transcription, Transcriptomics,
        TimeCourse, Regression, RNASeq, Normalization, WorkflowStep
License: GPL-2
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/RNAdecay
git_branch: devel
git_last_commit: 03eeb94
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:52:35 UTC; biocbuild
Author: Reed Sorenson [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8650-0601>),
  Katrina Johnson [aut],
  Frederick Adler [aut],
  Leslie Sieburth [aut] (ORCID: <https://orcid.org/0000-0001-6691-2000>)
Maintainer: Reed Sorenson <reedssorenson@gmail.com>

# Generated by roxygen2: do not edit by hand

export(a_high)
export(a_low)
export(aic)
export(aicc)
export(b_low)
export(cols)
export(comb_cv)
export(const_decay)
export(constraint_fun_list_maker)
export(decay_plot)
export(decaying_decay)
export(fit_var)
export(group_map)
export(groupings)
export(hl_plot)
export(log_lik)
export(mod_optimization)
export(n_par)
export(plain_theme)
export(sse_null_const_decay)
export(sse_null_decaying_decay)
useDynLib(general_Exp_1sse)
useDynLib(general_Exp_2sse)
useDynLib(general_Exp_3sse)
useDynLib(general_Exp_4sse)
useDynLib(general_dExp_1sse)
useDynLib(general_dExp_2sse)
useDynLib(general_dExp_3sse)
useDynLib(general_dExp_4sse)

# RNAdecay
RNAdecay is an R package for modeling RNA abundance decrease after inhibition of transcription. Estimates for decay rate and decaying decay rate parameters are generated by maximum likelihood optimization for each gene/transcript. Up to four treatments can be compared with each other. This is accomplished by modeling all possible treatment effects on decay and decaying decay rates using separate models or model constraints. Best models are selected using AICc comparison. Workflows and functions are provided for decay data normalization, modeling, and visualization. This software was developed as part of a study of RNA decay pathway contributions to decay genome-wide in _Arabidopsis thaliana_ ([Sorenson et al., 2018, Proceedings of the National Academy of Sciences 201712312](https://doi.org/10.1073/pnas.1712312115)).

C++ compilation is required upon installation from source. Modeling makes use of the functionality of the TMB package to generate high performance dynamically linked libraries from objective functions coded in C++ for efficient computation. C++ source files for these functions are provided but must be compiled. Compiling C++ code requires a compiler be installed separatedly on your system (e.g., Rtools34 or later for Windows,  https://cran.r-project.org/bin/windows/Rtools/; Xcode comand line tools for Mac, http://railsapps.github.io/xcode-command-line-tools.html; R development package for Linux, r-base-dev). 

To install and use `RNAdecay` use the following commands on the R console: 
```{r}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("RNAdecay")
```

Alternatively, to install from github (requires compiling, see above) use:

```{r}
install.packages("devtools")
devtools::install_github("reedssorenson/RNAdecay",type = "source")
library(RNAdecay)
```

################################################################################
#' model optimization for fitting exponential decay models to normalized data
#'
#' The mod_optimization function finds the estimates of model parameters by
#'   maximum likelihood, for a single gene on a specified list of models, and
#'   saves a tab delimited text file of the results named,'
#'   [geneID]_results.txt'.
#' The function does the following for each gene:
#' (1) it calculates log likelihood for each point in a 2 dimensional grid of
#'   evenly spaced alpha and beta values within the alpha and beta bounds
#'   specified using the null model (in which all treatment alphas are
#'   equivalent and all betas are equivalent).
#' (2) it calculates log likelihood for each point in a 1 dimensional range of
#'   evenly spaced alpha values within the alpha bounds using the single
#'   exponential null model (in which all treatment alphas are equivalent).
#' (3) For each of the grid points with the highest log likelihood from steps
#'   (1) and (2) 25 starting parameter value sets that are normally
#'   distributed around these points are generated.
#' (4) Parameter values are optimized for maximum likelihood using each of
#'   these 50 starting parameter sets using pre-compiled C++ functions loaded
#'  from dynamically linked libraries stored in the package on all models
#'  specified in the models argument.
#' (5) evaluates parameter estimates of all 50 optimizations based on the
#'   reported maximum liklihood upon convergence. Only parameter estimates
#'   that converged on the same and highest maximum likelihood are returned.
#' (6) returns the optimized parameter estimates, with model selection
#'   statistics.
#'
#' @param gene geneID from \code{data} to be modeled
#' @param data decay data data.frame with columns named 'geneID', 'treatment',
#'   't.decay', 'rep', 'value.'
#' @param models vector spceifying which models to run optimization on (e.g.,
#'   c('mod1', 'mod239'))
#' @param alpha_bounds vector of length 2 with lower and upper bounds for
#'   alpha
#' @param beta_bounds vector of length 2 with lower and upper bounds for beta
#' @param group grouping matrix for alphas or betas
#' @param mod data.frame specifying alpha and beta group pairs for each model
#' @param file_only logical; should output only be written to file (TRUE) or
#'   also return a data.frame of the results (FALSE)
#' @param path specify folder for output to be written
#'
#' @useDynLib general_dExp_4sse
#' @useDynLib general_Exp_4sse
#' @useDynLib general_dExp_3sse
#' @useDynLib general_Exp_3sse
#' @useDynLib general_dExp_2sse
#' @useDynLib general_Exp_2sse
#' @useDynLib general_dExp_1sse
#' @useDynLib general_Exp_1sse
#'
#' @export
#'
#' @return returns (if \code{file_only = FALSE}) and writes to \code{path} a
#'   data frame of model optimization results for \code{models} one row for
#'   each for \code{gene} using values for it found in \code{data}, the columns
#'   of the data frame are:
#' geneID, mod (model), model estimates [alpha_treatment1, ...,
#'   alpha_treatmentn, beta_treatment1, ..., beta_treatmentn, sigma2],
#'  logLik (maximum log likelihood), nPar (number of parameters in the model),
#'  nStarts (number of parameter starting value sets (of 50) that converged on
#'  a maximum likelihood peak), J (number of parameter starting value sets that
#'  converged on the highest - within 1e-4 - maximum likelihood of all
#'  parameter starting value sets), range.LL (range of maximum likelihoods
#'  values reached by algorithm convergence from all parameter starting value
#'  sets), nUnique.LL (number of unique maximum likelihoods values reached by
#'  algorithm convergence from all parameter starting value sets), C.alpha (sum
#'  of all coefficients of variation for each column of alpha estimates),
#'  C.beta (sum of all coefficients of variation for each column of beta
#'  estimates), C.tot (C.alpha+C.beta), AICc (calculated from the single
#'  highest maximum likelihood of all parameter starting value sets), AICc_est
#'  (calculated from the log likelihood value computed by using the mean of
#'  each parameter from all optimizations that converged on the highest
#'  maximum likelihood of all starting parameter value sets.)
#'
#' @examples
#'
#' mod_optimization(gene = 'Gene_BooFu',
#'                 data = data.frame(geneID=rep('Gene_BooFu',30),
#'                             treatment=c(rep('WT',15),rep('mut',15)),
#'                             t.decay=rep(c(0,7.5,15,30,60),6),
#'                             rep=rep(paste0('rep',c(rep(1,5),rep(2,5),rep(3,5))),2),
#'                             value= c(0.9173587, 0.4798672, 0.3327807, 0.1990708, 0.1656554,
#'                                      0.9407511, 0.7062988, 0.3450886, 0.3176824, 0.2749946,
#'                                      1.1026497, 0.6156978, 0.4563346, 0.2865779, 0.1680075,
#'                                      0.8679866, 0.6798788, 0.2683555, 0.5120951, 0.2593122,
#'                                      1.1348219, 0.8535835, 0.6423996, 0.5308946, 0.4592902,
#'                                      1.1104068, 0.5966838, 0.3949790, 0.3742632, 0.2613560)),
#'                 alpha_bounds = c(1e-4,0.75),
#'                 beta_bounds = c(1e-3,0.075),
#'                 models = 'mod1',
#'                 group = t(matrix(c(1,2,1,1,NA,NA),nrow=2,
#'                           dimnames=list(c('treat1','treat2'),c('mod1','mod2','mod3')))),
#'                 mod = as.data.frame(t(matrix(c(1,1,1,2,1,3,2,1,2,2,2,3),nrow=2,
#'                         dimnames=list(c('a','b'),paste0('mod',1:6))))),
#'                 file_only = FALSE,
#'                 path = paste0(tempdir(),"/modeling results"))
#'



mod_optimization <- function(gene,
                           data,
                           alpha_bounds,
                           beta_bounds,
                           models,
                           group,
                           mod,
                           file_only = TRUE,
                           path = "modeling_results") {
  if (!file.exists(path)) {
    dir.create(path)
  }

  genoSet <- 1:(length(unique(data$rep)) * length(unique(data$t.decay)))
  nTreat <- length(unique(data$treatment))
  nSet <- length(genoSet) * nTreat

  if (nTreat > 4) stop("mod_optimization can only handle up to 4 treatments.")

  gene <- as.character(gene)

  # pull out the data the specific gene data
  gdata <- data[data$geneID == as.character(gene),]
  t <- gdata$t.decay
  m <- gdata$value
  const <- constraint_fun_list_maker(mods = mod, groups = group)
  eps <- 1e-04

  #### Determine starting points. ####

  ### Define grid for evaluating model 240 while determining starting points.
  A <- seq(0, alpha_bounds[2], by = 0.001)
  ### Define grid for evaluating model 239 while determining starting points.
  a <- seq(alpha_bounds[1], alpha_bounds[2], by = 0.001)
  b <- seq(beta_bounds[1], beta_bounds[2], by = 0.001)

  ### Pick 25 points near the 'peak' of Model null double exponential model.  Find the
  ### SSE surface for the double exponential model...
  sse.nulldExp <- sapply(b, function(x) {
    sapply(
      a,
      FUN = sse_null_decaying_decay,
      b = x,
      m = m,
      t = t
    )
  })

  ### Find the location of the minimum.
  loc239 <-
    which(sse.nulldExp == min(sse.nulldExp), arr.ind = TRUE)

  ### Pick 25 points near the 'peak' of Model 240. Find the SSE surface for model
  ### 240...
  sse.nullExp <- sapply(A, FUN = sse_null_const_decay, m = m, t = t)
  ### Find the location of the minimum.
  loc240 <- which(sse.nullExp == min(sse.nullExp))

  ### Create the matrix of starting points. Pick the alpha values.
  aX0 <-
    matrix(c(
      stats::rnorm(25 * nTreat, mean = a[loc239[1, "row"]], sd = 0.01),
      stats::rnorm(25 * nTreat, mean = A[loc240], sd = 0.01)
    ), ncol = nTreat)
  ### Fix the alpha values outside of [1e-4,0.75]
  aX0 <- ifelse(aX0 < alpha_bounds[1], alpha_bounds[1], aX0)
  aX0 <- ifelse(aX0 > alpha_bounds[2], alpha_bounds[2], aX0)

  ### Pick the beta values.
  bX0 <-
    matrix(c(
      stats::rnorm(25 * nTreat, mean = b[loc239[1, "col"]], sd = 0.01),
      stats::runif(25 * nTreat, beta_bounds[1], beta_bounds[2])
    ), ncol = nTreat)
  ### Fix the beta values outside of [1e-3,0.075]
  bX0 <- ifelse(bX0 < beta_bounds[1], beta_bounds[1], bX0)
  bX0 <- ifelse(bX0 > beta_bounds[2], beta_bounds[2], bX0)

  ### Combine alpha and beta values into one matrix.
  X0 <- cbind(aX0, bX0)
  colnames(X0) <-
    c(paste0("alpha.int", 1:nTreat), paste0("beta.int", 1:nTreat))

  # set default parameters
  par.default <- c(rep(0, nTreat), rep(1, nTreat))
  names(par.default) <-
    c(paste0("a", 1:nTreat), paste0("b", 1:nTreat))


  # OPTIMIZATION OF THE DOUBLE EXPONENTIAL MODELS

  #### Create the objective function in R. ### This has to be done for each gene! ####
  obj.dExp <- TMB::MakeADFun(
    data = list(t = t, m = m),
    parameters = par.default,
    silent = TRUE,
    DLL = paste0("general_dExp_", nTreat, "sse")
    )

  results4ab <-
    sapply(models[models %in% rownames(mod)[mod$b != max(mod$b)]], function(x,
                                                                            eps,
                                                                            nSet,
                                                                            md,
                                                                            grp,
                                                                            alpha_bounds,
                                                                            beta_bounds) {
      fits <- apply(X0, 1, function(starts, y) {
        fit <-
          nloptr::slsqp(
            x0 = as.numeric(starts[1:ncol(X0)]),
            fn = obj.dExp$fn,
            gr = obj.dExp$gr,
            heq = const[[x]],
            lower = c(rep(alpha_bounds[1],
                          nTreat), rep(beta_bounds[1], nTreat)),
            upper = c(rep(alpha_bounds[2],
                          nTreat), rep(beta_bounds[2], nTreat))
          )
        unlist(c(
          geneID = gene,
          mod = x,
          as.numeric(fit$par),
          fit$value,
          fit$convergence
        ))
      }, y = x)

      fits <- data.frame(t(fits))
      fits[,-c(1:2)] <- sapply(fits[,-c(1, 2)], function(x)
        as.numeric(as.character(x)))
      fits <- as.data.frame(fits)
      colnames(fits) <- c("geneID", "mod", c(
        paste0("alpha_", as.character(unique(
          gdata$treatment
        ))),
        paste0("beta_", as.character(unique(
          gdata$treatment
        )))
      ), "SSE", "conv.code")

      fits <- fits[fits$conv.code == 4,]
      fits$sigma2 <- fit_var(sse = fits$SSE, n = nSet)
      fits$logLik <- log_lik(x = fits$SSE,
                           y = fits$sigma2,
                           n = nSet)

      max.LL <- max(fits$logLik)
      range.LL <- max.LL - min(fits$logLik)
      n.LL <- length(unique(round(fits$logLik, 4)))
      tmp <- fits[fits$logLik > (max.LL - eps),]
      C.alpha <- comb_cv(tmp[, grep("alpha", colnames(fits))])
      C.beta <- comb_cv(tmp[, grep("beta", colnames(fits))])
      C.tot <- C.alpha + C.beta
      par.est <- colMeans(tmp[, c(grep("alpha", colnames(fits)), grep("beta", colnames(fits)))])
      sigma2 <- mean(tmp$sigma2)
      nPar <- n_par(x, mod = md, group = grp)
      AICc <- aicc(max.LL, nPar, nSet)
      AICc_est <- aicc(log_lik(
        x = obj.dExp$fn(par.est),
        y = fit_var(sse = obj.dExp$fn(par.est),
                    n = nSet),
        n = nSet
      ),
      p = nPar,
      n = nSet)
      fit <- c(
        as.character(fits$geneID[1]),
        as.character(fits$mod[1]),
        par.est,
        sigma2,
        max.LL,
        nPar,
        nrow(fits),
        nrow(tmp),
        range.LL,
        n.LL,
        C.alpha,
        C.beta,
        C.tot,
        AICc,
        AICc_est
      )
      names(fit) <- c(
        "geneID",
        "mod",
        c(
          paste0("alpha_", as.character(unique(
            gdata$treatment
          ))),
          paste0("beta_", as.character(unique(
            gdata$treatment
          )))
        ),
        "sigma2",
        "logLik",
        "nPar",
        "nStarts",
        "J",
        "range.LL",
        "nUnique.LL",
        "C.alpha",
        "C.beta",
        "C.tot",
        "AICc",
        "AICc_est"
      )
      return(fit)
    }, eps = eps, nSet = nSet, grp = group, md = mod, alpha_bounds = alpha_bounds,
    beta_bounds = beta_bounds)
  results4ab <- as.data.frame(t(results4ab))
  results4ab[,-c(1, 2)] <- sapply(results4ab[,-c(1, 2)], function(x)
    as.numeric(as.character(x)))

  # OPTIMIZATION OF THE SINGLE EXPONENTIAL MODELS

  X0 <- aX0
  colnames(X0) <- c(paste0("alpha.int", 1:nTreat))

  obj.Exp <- TMB::MakeADFun(
    data = list(t = t, m = m),
    parameters = par.default[1:nTreat],
    silent = TRUE,
    DLL = paste0("general_Exp_", nTreat, "sse")
  )

  results4a <-
    sapply(models[models %in% rownames(mod)[mod$b == max(mod$b)]], function(x,
                                                                            eps, nSet, md, grp, alpha_bounds) {
      fits <- apply(X0, 1, function(starts, y) {
        fit <-
          nloptr::slsqp(
            x0 = as.numeric(starts[1:ncol(X0)]),
            fn = obj.Exp$fn,
            gr = obj.Exp$gr,
            heq = const[[x]],
            lower = rep(alpha_bounds[1], nTreat),
            upper = rep(alpha_bounds[2], nTreat)
          )
        unlist(c(
          geneID = gene,
          mod = x,
          as.numeric(fit$par),
          rep(0, nTreat),
          fit$value,
          fit$convergence
        ))
      }, y = x)

      fits <- data.frame(t(fits))
      fits[,-c(1:2)] <- sapply(fits[,-c(1, 2)], function(x)
        as.numeric(as.character(x)))
      fits <- as.data.frame(fits)
      colnames(fits) <- c("geneID", "mod", c(
        paste0("alpha_", as.character(unique(
          gdata$treatment
        ))),
        paste0("beta_", as.character(unique(
          gdata$treatment
        )))
      ), "SSE", "conv.code")

      fits <- fits[fits$conv.code == 4,]
      fits$sigma2 <- fit_var(sse = fits$SSE, n = nSet)
      fits$logLik <- log_lik(x = fits$SSE,
                           y = fits$sigma2,
                           n = nSet)

      max.LL <- max(fits$logLik)
      range.LL <- max.LL - min(fits$logLik)
      n.LL <- length(unique(round(fits$logLik, 4)))
      tmp <- fits[fits$logLik > (max.LL - eps),]
      C.alpha <- comb_cv(tmp[, grep("alpha", colnames(fits))])
      C.beta <- comb_cv(tmp[, grep("beta", colnames(fits))])
      C.tot <- C.alpha + C.beta
      par.est <- colMeans(tmp[, c(grep("alpha", colnames(fits)),
                                 grep("beta", colnames(fits)))])
      sigma2 <- mean(tmp$sigma2)
      nPar <- n_par(x, mod = md, group = grp)
      AICc <- aicc(max.LL, nPar, nSet)
      AICc_est <- aicc(log_lik(
        x = obj.Exp$fn(par.est[1:nTreat]),
        y = fit_var(sse = obj.Exp$fn(par.est[1:nTreat]),
                    n = nSet),
        n = nSet
      ),
      p = nPar,
      n = nSet)
      fit <- c(
        as.character(fits$geneID[1]),
        as.character(fits$mod[1]),
        par.est,
        sigma2,
        max.LL,
        nPar,
        nrow(fits),
        nrow(tmp),
        range.LL,
        n.LL,
        C.alpha,
        C.beta,
        C.tot,
        AICc,
        AICc_est
      )
      names(fit) <- c(
        "geneID",
        "mod",
        c(
          paste0("alpha_", as.character(unique(
            gdata$treatment
          ))),
          paste0("beta_", as.character(unique(
            gdata$treatment
          )))
        ),
        "sigma2",
        "logLik",
        "nPar",
        "nStarts",
        "J",
        "range.LL",
        "nUnique.LL",
        "C.alpha",
        "C.beta",
        "C.tot",
        "AICc",
        "AICc_est"
      )
      return(fit)
    }, eps = eps, nSet = nSet, grp = group, md = mod, alpha_bounds = alpha_bounds)
  results4a <- as.data.frame(t(results4a))
  results4a[,-c(1, 2)] <- sapply(results4a[,-c(1, 2)], function(x)
    as.numeric(as.character(x)))

  results <- rbind(results4a, results4ab)
  results <- results[order(as.numeric(gsub("mod", "", results$mod))),]

  utils::write.table(results, paste0(path, "/", gene, "_results.txt"), sep = "\t")
  cat(gene, "done     \n")  #; utils::timestamp()

  return(if (file_only)
    invisible(NULL)
    else
      results)

}

#' log likelihood
#'
#' Calculates the log likelihood value from the sum of the squared errors, sigma2, and the total number of data points.
#'
#' @param x is the minimum SSE from the slsqp() fitting
#' @param y is sigma2 cooresponding to the minimum SSE
#' @param n is the total number of observations of a single gene (e.g., 8 time points X 4 replicates X 4 treatments/genotypes = 128)
#'
#' @return returns Log Likelihood
#'
#' @export
#'
#' @keywords internal
#'
#' @examples
#' log_lik(1,1/128,128)


log_lik <- function(x, y, n) {
  -(n / 2) * log(2 * pi * y) - (1 / (2 * y)) * x
}

#' model color map
#'
#' group_map makes a color map of alpha and beta equivalence groups by model. Similar colors in a row indicate constrained parameter equivalence between treatements. Gray indicates values of 0.
#'
#' @param decaydata 5 column data.frame with colnames "geneID","treatment","t.decay","rep","value"
#' @param path write path and file name, must end in ".pdf"
#' @param nEquivGrp number of equivalence groups based on number of treatments
#' @param groups equivalence group matrix
#' @param mods alpha beta equivalence group usage index (matrix)
#'
#' @return creates a model colormap and writes it to a pdf file named \code{path}
#'
#' @export
#' @examples
#' group_map(decaydata=data.frame(geneID=paste0("gene",1:4),
#'                     treatment=as.factor(rep(paste0("treat",1:2),2)),
#'                     t.decay=0:3,
#'                     rep=rep("rep1"),
#'                     value=c(1,0.5,0.25,0.12)),
#'          path=paste0(tempdir(),"/parameter equivalence colormap.pdf"),
#'          nEquivGrp = 2,
#'          groups = t(matrix(c(1,2,1,1,NA,NA),nrow=2,
#'                     dimnames=list(c("treat1","treat2"),c("grp1","grp2","grp3")))),
#'          mods = t(matrix(c(1,1,1,2,1,3,2,1,2,2,2,3),nrow=2,
#'                          dimnames=list(c("a","b"),paste0("mod",1:6)))))


group_map <- function(decaydata,
                    path,
                    nEquivGrp = nEquivGrp,
                    groups = groups,
                    mods = mods) {
  nTreat <- length(unique(decaydata$treatment))
  groupingsA <- t(matrix(rep(groups[1,], nEquivGrp + 1), nrow = nTreat))
  for (i in 2:nEquivGrp) {
    groupingsA <- rbind(groupingsA, t(matrix(rep(
      groups[i,], nEquivGrp + 1
    ), nrow = nTreat)))
  }
  colnames(groupingsA) <- paste0("alpha_", unique(decaydata$treatment))
  groupingsB <- rbind(groups[1:nEquivGrp,] + nTreat, rep(NA, nTreat))
  rownames(groupingsB)[nEquivGrp + 1] <- paste0("grp", nEquivGrp + 1)
  groupingsB <- t(matrix(rep(t(groupingsB), nEquivGrp), nrow = nTreat))
  colnames(groupingsB) <- paste0("beta_", unique(decaydata$treatment))
  groupings <- cbind(groupingsA, groupingsB)
  rm(groupingsA, groupingsB)
  rownames(groupings) <- rownames(mods)
  grDevices::pdf(path, width = 8, height = 10)
  heats <- c("darkblue",
            "green",
            "orange",
            "yellow",
            "magenta",
            "cyan",
            "red",
            "darkgray")[1:(nTreat * 2)]
  gplots::heatmap.2(
    groupings,
    trace = "none",
    breaks = 1:(nTreat * 2 + 1) - 0.5,
    col = heats,
    Colv = FALSE,
    Rowv = FALSE,
    dendrogram = 'none',
    #labCol=T,labRow=T,
    main = "Model groupings - similar colors within\n each model have constrained equivalence",
    margins = c(10, 5),
    lwid = c(0.25, 0.75),
    lhei = c(0.20, 0.8),
    # relative width and height of the box with the key vs the box with the heatmap
    cexRow = 0.25,
    key = FALSE
  )
  grDevices::dev.off()
}

#' Indexes column names of a data.frame matching multiple patterns (i.e., multigrep)
#'
#' Identifies dataframe column names that have all of the pattern arguments .
#'
#' Be aware that column data labels that are part of another data label are not advisable (e.g. mut1, mut2, mut1.mut2; cols(df,'mut1') will return indices for both 'mut1' and 'mut1.mut2' labeled columns
#'
#' @param df a dataframe with column names to index
#' @param patterns character vector or vector of regular expressions passed to grep pattern argument
#' @param w,x,y,z (for backwards compatibility) separate arguments for patterns, if used patterns argument will be ignored
#'
#' @return  returns a vector of integer indices of the column names of \code{df} that match to all of \code{patterns}
#'
#' @export
#'
#' @examples
#' cols(df=data.frame(xyz=1:5,zay=6:10,ybz=11:15,tuv=16:20),patterns = c('y','z')) ## returns 1 2 3
#' cols(df=data.frame(xyz=1:5,zay=6:10,ybz=11:15,tuv=16:20), w = 'y', x = 'z') ## returns 1 2 3
#'

cols <- function(patterns,df,w=NA,x=NA,y=NA,z=NA) {

  # for backwards compatability
  if(any(!is.na(c(w,x,y,z)))){
  patterns <- c(w,x,y,z)
  patterns <- patterns[!is.na(patterns)]
  }

  column_names <- colnames(df)

  which(sapply(column_names, function(y) all(sapply(patterns, function(pat) grepl(pat,y)))))
  }

#' hl_plot() function
#'
#' Plots RNA half-life distribution with select half-lives of select RNAs as large arrows colored by treatment using the ggplot2 package.
#'



#' @param geneID single gene ID from data set (e.g. "AT3G14100") for which to plot data/model
#' @param gene_symbol (optional) pasted to gene ID in plot label (e.g., "AT3G14100/UBP1C)
#' @param df_decay_rates data.frame of modeling results with decay rate columns labeled as alpha_<treatment>
#' @param hl_dist_treatment name of the treatment for which the background distribution will be plotted
#' @param hl_treatment names of the treatments for which arrows indicating half-life will be plotted
#' @param arrow_colors (optional) character vector of R colors; named with corresponding treatments
#' @param arrow_lab_loc label arrows on plot ("plot") or in a key ("key")
#' @param x_limits x-axis (half-life) limits in min; default is log(2)/c(0.25,4.5e-4)
#' @param x_breaks x-axis (half-life) breaks/tick marks in min defaults to c(5,1:12*10,180,240,300,360,420,480,720,1080,1440)
#' @param x_tick_labels x-axis (half-life) break labels, defaults to c("5","10","","30","","","60","","","","","","2h","","4h","","","","8h","12h","","24h")+
#'


#'
#' @return returns a ggplot to be used with print; could also be modified using the syntax of ggplot2 e.g.'+geom_XXXX(...)'
#'
#' @export
#'
#' @examples
#'
#' p <- hl_plot(
#' geneID = rownames(RNAdecay::results)[4],
#' df_decay_rates = RNAdecay::results,
#' hl_treatment = c("WT","sov","vcs","vcs.sov"),
#' hl_dist_treatment = "WT",
#' arrow_colors = c(WT = "#88CCEE", sov = "#CC6677", vcs = "#117733",vcs.sov = "#882255"),
#' arrow_lab_loc = "key",
#' gene_symbol = ""
#' )
#'
#'print(p)
#'
#' p <- hl_plot(
#' geneID = rownames(RNAdecay::results)[4],
#' gene_symbol = "",
#' df_decay_rates = RNAdecay::results,
#' hl_dist_treatment = "WT",
#' hl_treatment = c("WT","sov","vcs","vcs.sov"),
#' arrow_colors = c(WT = "#88CCEE", sov = "#CC6677", vcs = "#117733",vcs.sov = "#882255"),
#' arrow_lab_loc = "plot"
#' )
#'
#'print(p)


hl_plot <- function (
    geneID,
    gene_symbol = "",
    df_decay_rates,
    hl_dist_treatment,
    hl_treatment,
    arrow_colors = NA,
    arrow_lab_loc = c("key"),
    x_limits = log(2)/c(0.25, 0.00045),
    x_breaks = c(5, 1:12 * 10, 180, 240, 300, 360, 420, 480, 720, 1080, 1440),
    x_tick_labels = c("5", "10", "", "30", "", "", "60", "", "", "", "", "", "2h",
                      "", "4h", "", "", "", "8h", "12h", "", "24h")
)
{
  treatment.cols <- paste0("alpha_", hl_treatment)
  names(treatment.cols) <- hl_treatment
  hls = round(log(2)/df_decay_rates[geneID, treatment.cols],
              1)
  if (all(is.na(arrow_colors))) {
    arrow_colors <- c("#88CCEE", "#CC6677", "#DDCC77", "#117733",
                      "#332288", "#AA4499", "#44AA99", "999933", "#882255",
                      "#661100", "#6699CC")[seq_along(hl_treatment)]
    names(arrow_colors) = hl_treatment

    }

    if(!hl_dist_treatment %in% hl_treatment) {stop("'hl_dist_treatment' must be one of 'hl_treatment'.")}

    if(length(geneID)>1) {

      warning("'length(geneID)' > 1, only the first will be used.")
      geneID <- geneID[1]

      }

    if(all(is.na(hls[geneID,]))){

      stop("'geneID' is not found in 'df_decay_rates'.")

      }

    if(length(hl_treatment) != length(arrow_colors)) {

      stop("'length(hl_treatment)' does not equal 'length(arrow_colors'. These must be the same.")

      }

    if(is.null(names(arrow_colors)) | !all(names(arrow_colors) %in% hl_treatment)) {

      warning("Not all colors' names match hL_treatment or names are missing. Colors will be arbitrarily assigned.")
      names(arrow_colors) <- hl_treatment

      }

    labs <- sapply(hl_treatment, function(x){

      hl <- hls[geneID,treatment.cols[x]]

      if(hl > max(x_breaks)){

        paste0("paste(\"",x," \",italic(t)[\"1/2\"] > \"24 h\")")

      } else{

        paste0("paste(\"",x," \",italic(t)[\"1/2\"] == \"",

               if(hl < 120)    {paste0(as.character(round(hl))," min")}    else    {paste0(round(hl/60,1)," h")},

               "\")")}

    })

    hl_dist <- df_decay_rates[df_decay_rates[,treatment.cols[hl_dist_treatment]] > 4.8e-5,
                              treatment.cols[hl_dist_treatment]]
    quants <- round(stats::quantile(log(2)/hl_dist))
    peak <- max(table(cut(log10(log(2)/hl_dist),include.lowest = TRUE,
                          breaks = c(seq(from =log10(x_limits[1]),
                                         to = x.max<-if(max(x_breaks) < max(log(2)/hl_dist)) {
                                           log10(max(x_limits))}    else    {max(log10(log(2)/hl_dist))},
                                         by = 0.02)))))

    arrow_lengths <- peak*0.9^seq_along(hl_treatment)
    names(arrow_lengths) <- hl_treatment
    arrow_label_heights <- arrow_lengths + arrow_lengths*0.1
    names(arrow_label_heights) <- hl_treatment
    arrow_thickness <- 3*0.85^seq_along(hl_treatment)
    names(arrow_thickness) <- hl_treatment
    arrowhead_length_in <- grDevices::dev.size(units = "in")[2]*0.04*0.85^seq_along(hl_treatment)
    names(arrowhead_length_in) <- hl_treatment

    p <- ggplot2::ggplot(df_decay_rates[df_decay_rates[,treatment.cols[hl_dist_treatment]] > 4.8e-5,])

    # add histogram and x scale
    p <- p + ggplot2::geom_histogram(data = data.frame(hl_dist = hl_dist),
                                     mapping = ggplot2::aes(x = log(2)/hl_dist),
                                     binwidth=0.02,
                                     fill = grDevices::gray(0.8))+
      ggplot2::geom_vline(xintercept=quants[2],color = "white",linetype = "dashed",linewidth = 0.25)+
      ggplot2::geom_vline(xintercept=quants[3],color = "white",linetype = "dashed",linewidth = 0.25)+
      ggplot2::geom_vline(xintercept=quants[4],color = "white",linetype = "dashed",linewidth = 0.25)+
      ggplot2::scale_x_log10(limits=x_limits,
                             breaks = x_breaks,
                             labels = x_tick_labels)+
      ggplot2::annotate(geom = "text",
                        x = max(quants[3]),
                        y = 0,
                        label = paste0(hl_dist_treatment," distribution"),
                        hjust = 0.575,
                        vjust = -0.2,
                        color = grDevices::gray(0.65),
                        size = 72.27*grDevices::dev.size(units = "in")[2]/60)

    if(arrow_lab_loc == "plot"){

      for(i in hl_treatment){

        # add arrow
        hl <- log(2)/df_decay_rates[geneID,treatment.cols[i]]
        p <- p+ggplot2::geom_segment(data = data.frame(x=if(hl > x_limits[2]) {x_limits[2]} else {hl},
                                                       y=arrow_lengths[i]),#arrow height
                                     mapping = ggplot2::aes(x = x, y=y,xend = x, yend = 0),
                                     arrow = ggplot2::arrow(angle = 18,
                                                         length = grid::unit(arrowhead_length_in,"in"),
                                                         type = "closed"),#arrow head
                                     color = arrow_colors[i],
                                     linewidth = arrow_thickness[i],#arrow width in pts
                                     lineend = "butt",
                                     linejoin = "mitre")

        # add arrow label
        p <- p+ggplot2::annotate(parse = TRUE,geom = "text",
                                 x = if(hl>max(x_limits[2])){max(x_limits[2])}else{hl},
                                 y = arrow_label_heights[i], # arrow lebel height
                                 label = labs[i],
                                 hjust = if(hl>103.4){0.9} else {0.15},
                                 vjust = 0.5,
                                 color = arrow_colors[i],
                                 size = 72.27*grDevices::dev.size(units = "in")[2]/75)
      }
    }

    if(arrow_lab_loc == "key"){

      hl <- log(2)/df_decay_rates[geneID,treatment.cols]

      p <- p + ggplot2::geom_segment(data = data.frame(x=unlist(hl),
                                                       y=arrow_lengths[hl_treatment], #arrow height
                                                       labs = hl_treatment),
                                     mapping = ggplot2::aes(x = x, y=y,xend = x, yend = 0,
                                                            color = labs,
                                                            linewidth = labs),
                                     arrow = ggplot2::arrow(angle = 18,
                                                         length = grid::unit(arrowhead_length_in,"in"),
                                                         type = "closed"), #arrow head
                                     lineend = "butt",
                                     linejoin = "mitre")+
        ggplot2::scale_color_manual(NULL,values = arrow_colors,breaks = hl_treatment,labels=scales::parse_format()(labs[hl_treatment]))+
        ggplot2::scale_size_manual(values = arrow_thickness)+
        ggplot2::guides(size="none")
    }

    p <- p + ggplot2::annotate(geom="text",
                               label = if(gene_symbol != "") {paste0(geneID,"/",gene_symbol)} else {geneID},
                               x= 10^x.max,
                               y = peak*1.25,
                               hjust = 1,
                               vjust = 1,
                               color = "black",
                               size = 72.27*grDevices::dev.size(units = "in")[2]/50)+
      ggplot2::xlab(expression(italic(t)["1/2"]))+
      ggplot2::coord_cartesian(xlim = x_limits,
                               ylim = c(0,peak*1.25))+
      ggplot2::theme_classic()+
      ggplot2::theme(title = ggplot2::element_text(color = "darkblue"),
                     axis.text = ggplot2::element_text(color = grDevices::gray(0)),
                     axis.title = ggplot2::element_text(color = grDevices::gray(0))
      )
}

#' exponential decay functions
#'
#' Constant decay rate function (const_decay(), case when betas=0) e^-a*t; decaying decay rate function (decaying_decay())  e^(-(a/b)*(1-e^(-b*t))). Functions are normalized so at t=0 the function is 1.
#'
#' @param t time (in minutes)
#' @param a alpha (in per time, thus in per minute when time is in minutes)
#' @param par vector of length 2 containing alpha (par[1]) and beta (par[2]) values; alpha=initial decay rate, beta=decay of decay rate (both in per time, thus in per minute when time is in minutes)
#'
#' @return returns abundance after time \code{t}  at alpha initial decay rate and beta decay of decay rate relative to an initial abundance of 1
#'
#' @export
#'
#' @examples
#' const_decay(10,log(2)/10) ## returns 0.5
#' decaying_decay(10,c(log(2)/10,0.01)) ##returns 0.5170495

const_decay <- function(t, a) {
  exp(-a * t)
}


#' @rdname const_decay
#' @export
decaying_decay <- function(t, par) {
  a <- par[1]
  b <- par[2]
  exp(-(a / b) * (1 - exp(-b * t)))
}

#' a custom ggplot2 theme
#'
#' A custom ggplot2 theme generating function for ggplot2 plots; can be further manipulated using standard ggplot2 syntax.
#'
#' @param bigFont larger font size of axis labels in points (used for plot title, axis titles, facet titles)
#' @param smFont fractional mulitiplier of \code{bigFont} (used for axis text)
#' @param x.ang x-axis label angle
#' @param leg.pos legend position on plot as relative coordinates c(x,y) (i.e., range is [0,1]) or 'right', 'left', 'above', 'below'
#'
#' @return returns a ggplot2 theme of class "theme" "gg"
#'
#' @export
#'
#' @examples plain_theme(10)


plain_theme <-
  function (bigFont = 30, smFont = 0.85, x.ang = 0, leg.pos = c(0.85, 0.85))
  {
    ggplot2::theme(panel.background = ggplot2::element_blank(),
                   plot.title = ggplot2::element_text(size = bigFont, vjust = 0.5),
                   panel.grid.minor = ggplot2::element_blank(),
                   axis.line.x = ggplot2::element_line(colour = "black", linewidth = 1, linetype = "solid"),
                   axis.line.y = ggplot2::element_line(colour = "black", linewidth = 1, linetype = "solid"),
                   plot.margin = grid::unit(c(1, 1, 1, 1), "lines"),
                   axis.title.x = ggplot2::element_text(size = bigFont),
                   axis.text.x = ggplot2::element_text(colour = "black", size = bigFont * smFont,
                                                       angle = x.ang, vjust = 0.5),
                   axis.title.y = ggplot2::element_text(size = bigFont, angle = 90),
                   axis.text.y = ggplot2::element_text(colour = "black",size = bigFont * smFont),
                   legend.box.background = ggplot2::element_blank(),
                   legend.key = ggplot2::element_rect(colour = "white", fill = "white"),
                   legend.position = leg.pos,
                   legend.text = ggplot2::element_text(size = bigFont, face = c("italic")),
                   legend.title = ggplot2::element_text(size = bigFont),
                   legend.key.height = grid::unit(bigFont/12, "lines"),
                   legend.title.align = 0.1,
                   strip.text.x = ggplot2::element_text(size = bigFont, colour = "black", face = "bold"),
                   strip.text.y = ggplot2::element_text(size = bigFont,
                                                        colour = "black", face = "bold", angle = 270))
  }

#' Example double exponential decay modeling results
#'
#' Example results from maximum likelihood modeling of double exponential RNA decay of 118 genes.
#'
#' @format a list of data frames, each with 240 rows (1/model) with 22 columns and 240 rows.
#'  \describe{
#'     \item{geneID}{gene identifier}
#'     \item{mod}{model names as factors}
#'     \item{alpha_XXX}{decay rate estimate of genotype XXX, in per time (min^-1)}
#'     \item{beta_XXX}{decay of decay rate estimate of genotype XXX, in per time (min^-1)}
#'     \item{sigma2}{variance estimate}
#'     \item{logLik}{maxium log likelihood}
#'     \item{nPar}{number of parameters in the given model}
#'     \item{nStarts}{number of parameter starting value sets (of 50) that converged on a maximum likelihood peak}
#'     \item{J}{number of parameter starting value sets that converged on the highest - within 1e-4 - maximum likelihood of all parameter starting value sets}
#'     \item{range.LL}{range of maximum likelihoods values reached by algorithm convergence from all parameter starting value sets}
#'     \item{nUnique.LL}{number of unique maximum likelihoods values reached by algorithm convergence from all parameter starting value sets}
#'     \item{C.alpha}{sum of all coefficients of variation for each column of alpha estimates}
#'     \item{C.beta}{sum of all coefficients of variation for each column of beta estimates}
#'     \item{C.tot}{C.alpha+C.beta}
#'     \item{AICc}{calculated from the single highest maximum likelihood of all parameter starting value sets}
#'     \item{AICc_est}{calculated from the log likelihood value computed by using the mean of each parameter from all optimizations that converged on the highest maximum likelihood of all starting parameter value sets}
#'     }
#'
#' @source Sorenson et al. (2017) Submitted; \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE86361}
#'
"models"

#' sum of the squared errors for null models
#'
#' For a model that uses a constant decay rate or a decaying decay rate, calculates the sum of the squared errors (differences between the supplied data points and the modeled values based on alpha and/or beta values). For these models all treatments are assumed to have the same \code{a} (alpha) and/or \code{b} (beta).
#'
#' @param a alpha value
#' @param b beta value
#' @param m mRNA abundance values
#' @param t time points of \code{m}
#'
#' @return  Returns the sum of the squared errors
#'
#' @export
#'
#' @keywords internal
#'
#' @examples
#' sse_null_decaying_decay(a=0.05, b = 0.001,
#'            m = c(1,1,1,0.99,0.5,0.5,0.5,0.49,0.25,0.25,0.25,0.24,0.12,0.125,0.125,0.126),
#'            t = rep(c(0,10,20,30),each = 4))
#' sse_null_const_decay(a=0.05,
#'            m = c(1,1,1,0.99,0.5,0.5,0.5,0.49,0.25,0.25,0.25,0.24,0.12,0.125,0.125,0.126),
#'            t = rep(c(0,10,20,30),each = 4))


sse_null_decaying_decay <- function(a, b, m, t) {
  sum((m - exp(-(a / b) * (1 - exp(
    -b * t
  )))) ^ 2)
}

#' @rdname sse_null_decaying_decay
#' @export
sse_null_const_decay <- function(a, m, t) {
  sum((m - exp(-a * t)) ^ 2)
}

#' Akaike information criterion (with correction)
#'
#' Calculates AIC or AICc.
#'
#' @param maxlLik maximum log likelihood value identified upon model convergence
#' @param p number of parameters in the model
#' @param n is the total number of observations of a single gene (e.g., 8 time
#'   points X 4 replicates X 4 treatments/genotypes = 128)
#'
#' @return returns the AIC or AICc values
#'
#' @export
#'
#' @keywords internal
#'
#' @examples
#' aicc(100,5,15)


aic <- function(maxlLik, p) {
  2 * p - 2 * maxlLik
}

#' @rdname aic
#' @export
aicc <- function(maxlLik, p, n) {
  2 * p - 2 * maxlLik + ((2 * p * (1 + p)) / (n - p - 1))
}

#' sigma^2 estimation
#'
#' Calculates the variance (sigma^2) estimate from the sum of the squared errors from the fit model.
#'
#' @param sse is the minimum SSE (sum of the squared errors) from the slsqp() fitting
#' @param n is the total number of observations of a single gene (e.g., 8 time points X 4 replicates X 4 treatments/genotypes = 128)
#'
#' @return returns the sigma squared estimate
#'
#' @export
#'
#' @keywords internal
#'
#' @examples
#' fit_var(1,128)


fit_var <- function(sse, n) {
  sse / n
}

#' combined adjusted coefficient of variation
#'
#' Calculates the sum of the column standard devation divided by the sum of the column mean and a small value to avoid dividing by 0 (\code{eps})
#'
#' @param X data.frame or matrix of numeric data
#' @param eps small value to add to the mean to avoid dividing by 0; defaults to 1e-4
#'
#' @return returns the sum of the coefficients of variation for all columns of \code{X}
#'
#' @export
#'
#' @keywords internal
#'
#' @examples
#' comb_cv( data.frame( test1=rep(0,5), test2=c(0.2,0.3,0.35,0.27,0.21) ) )

comb_cv <- function(X, eps = 1e-04) {
  Y <- as.matrix(X)
  sd.Y <- apply(Y, 2, stats::sd)
  mean.Y <- apply(Y, 2, mean)
  return(sum(sd.Y / (mean.Y + eps)))
}

#' Example double exponential decay modeling results
#'
#' Example results from maximum likelihood modeling of double exponential RNA decay of 118 genes. Results include parameter estimates, selected model, and alpha and beta groupings.
#'
#' @format a data frame with 18 columns and 118 rows.
#'  \describe{
#'     \item{alpha_XXX}{decay rate estimate of genotype XXX, in per time (min^-1)}
#'     \item{beta_XXX}{decay of decay rate estimate of genotype XXX, in per time (min^-1)}
#'     \item{sigma2}{variance estimate}
#'     \item{model}{selected model number}
#'     \item{alpha_grp}{model alpha grouping number }
#'     \item{beta_grp}{model beta grouping number }
#'     \item{alpha_subgroup}{model alpha subgroup number}
#'     \item{alphaPattern}{model alpha subgroup pattern; i.e. order of genotypes of increaseing decay rate}
#'     \item{betaPattern}{model beta subgroup pattern; i.e. order of genotypes of increaseing decay of decay rate}
#'     \item{rA_XXX}{relative alpha value of genotype XXX compared to WT}
#'     \item{nEqMods}{number of models that were not different than the selected model based on a AICc difference <2}
#'     \item{nEqAgp}{number of alpha groups represented in nEqMods}
#'     }
#'
#' @source Sorenson et al. (2017) Submitted; \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE86361}
#'
"results"

#' decay_plot() function
#'
#' Plots RNA decay data and/or decay models using the ggplot2 package.
#'
#' @param geneID single gene ID from data set (e.g. "AT1G00100") for which to plot data/model
#' @param xlim,ylim vector of length 2 defineing the limits of the plot (zooms in on data)
#' @param xticks,yticks vectors specifyng tick marks for the x and y axes
#' @param alphaSZ text size of alpha and beta parameter labels if plotted
#' @param what character vector specifying what to plot; any or all (default) of "Desc","models","reps","meanSE","alphas&betas"
#'    "Desc"         - plots gene descriptions behind data
#'    "models"       - plots the selected fit model
#'    "reps"         - plots individual replicate data as distinct shapes
#'    "meanSE        - plots the replicate means and standard errors
#'    "alphas&betas" - plots the values of the alphas and betas for each model below the model at the greatest x position
#' @param treatments what treatments/genotypes to plot from the supplied data
#' @param colors vector of R recognized colors (e.g. "red","darkblue")
#' @param DATA (required) normalized abundance decay data with column names: "geneID", "treatment","t.decay", "rep","value"
#' @param mod.results (optional; required for plotting models) data.frame of the model results as output from the modeling (e.g. "alphas+betas+mods+grps+patterns+relABs.txt")
#' @param gdesc (optional; required for plotting gene descriptions) gene descriptions (geneID-named vector of gene descriptions geneID must match those of data)
#' @param desc.width width of gene descriptions (in number of characters) before word wrap
#'
#' @return returns a ggplot to be used with print; could also be modified using the syntax of ggplot2 e.g.'+geom_XXXX(...)'
#'
#' @export
#'
#' @examples
# p<-RNAdecay::decay_plot(
#   geneID = "GOI_id",
#   mod.results = data.frame(alpha_WT = 0.0830195, beta_WT = 0.04998945,
#                            model = 1, alpha_grp = 1, beta_grp = 1, alpha_subgroup = 1.1,
#                            row.names = "GOI_id"),
#   what = c("meanSE","alphas&betas","models"),
#   treatments = "WT",
#   colors = "black",
#   DATA = data.frame(geneID=rep("GOI_id",15),
#                     treatment=rep("WT",15),
#                     t.decay=rep(c(0,7.5,15,30,60),3),
#                     rep=paste0("rep",c(rep(1,5),rep(2,5),rep(3,5))),
#                     value= c(0.9173587, 0.4798672, 0.3327807, 0.1990708, 0.1656554,
#                              0.9407511, 0.7062988, 0.3450886, 0.3176824, 0.2749946,
#                              1.1026497, 0.6156978, 0.4563346, 0.2865779, 0.1680075)),
#   xlim = c(0, 65),
#   alphaSZ = 10)
# print(p)


decay_plot <-
function (geneID,
          xlim = c(0, 500),
          ylim = c(0, 1.25),
          xticks = NA,
          yticks = 0:5/4,
          alphaSZ = 8,
          what = c("Desc", "models", "reps", "meanSE", "alphas&betas"),
          DATA,
          treatments = NA,
          colors = NA,
          mod.results = NA,
          gdesc = NA,
          desc.width = 55)
{
if (any(!geneID %in% rownames(mod.results))) {
  stop(paste0("geneID:", geneID, " not found in the dataset."),
       call. = FALSE)
}
if (any(!treatments %in% gsub("alpha_", "", colnames(mod.results)[grep("alpha_",
                                                                       colnames(mod.results)[1:4])]))) {
  stop(paste0("Supplied 'treatments' are not found in 'mod.results'."))
}
if (any(!treatments %in% unique(DATA$treatment))) {
  stop(paste0("'treatments' indicated are not found in 'DATA'."))
}
dExp <- function(t, par) {
  a <- par[1]
  b <- par[2]
  exp(-(a/b) * (1 - exp(-b * t)))
}
fun_exp <- function(t, a) {
  exp(-a * t)
}
wrapper <- function(x, ...) {
  paste(strwrap(x, ...), collapse = "\n")
}
if (any(what %in% "models"))
  mod <- mod.results[geneID, "mod"] else mod <- NA
if (any(what %in% "models"))
  A_grp <- mod.results[geneID, "alpha_subgroup"] else ""
if (is.na(treatments[1]))
  treatments <- unique(DATA$treatment) else treatments <- treatments[treatments %in% unique(DATA$treatment)]
if (is.na(colors[1]))
  colors <- grDevices::rainbow(length(treatments), alpha = 1)
if (any(is.na(xticks)))
  xticks <- c(0, 1:5 * diff(xlim)/5 + xlim[1])
names(colors) <- treatments
p <- ggplot2::ggplot(data = DATA[DATA$geneID == geneID &
                                   DATA$treatment %in% treatments, ])
if (any(what %in% "Desc")) {
  p <- p + ggplot2::annotate(geom = "text", x = xlim[2]/2,
                             y = ylim[2], label = wrapper(paste0(geneID, if (any(what %in%
                                                                                 "models")) {
                               paste0(" - model ", mod)
                             }
                             else {
                               ""
                             }, paste0(" - ", gdesc[geneID])), desc.width), vjust = 1,
                             hjust = 0.5, color = grDevices::gray(0.4), size = alphaSZ *
                               0.4, family = c("mono"), fontface = "plain",
                             angle = 0)
}
else {
  p <- p + ggplot2::ggtitle(if (any(what %in% "models")) {
    paste0(geneID, " - model ", mod)
  }
  else {
    geneID
  })
}
if (any(what %in% "alphas&betas")) {
  p <- p + ggplot2::geom_text(parse = TRUE,
                              data = data.frame(treatment = treatments,
                                                text = sapply(treatments, function(g) {
                                                  paste0("alpha==", as.character(round(mod.results[geneID,
                                                                                                   paste0("alpha_", g)], 4)), "~beta==", as.character(round(mod.results[geneID,
                                                                                                                                                                        paste0("beta_", g)], 4)))
                                                }), x = rep(xlim[2], length(treatments)), y = if (any(mod.results[geneID,
                                                                                                                  (length(unique(DATA$treatment)) + 1):(length(unique(DATA$treatment)) *
                                                                                                                                                        2)] == 0)) {
                                                  sapply(treatments, function(g) {
                                                    c(max(fun_exp(xlim[2], unlist(mod.results[geneID,
                                                                                              paste0("alpha_", g)])) - 0.2, 0))
                                                  })
                                                }
                                                else {
                                                  sapply(treatments, function(g) {
                                                    c(max(dExp(xlim[2], unlist(mod.results[geneID,
                                                                                           paste0(c("alpha_", "beta_"), g)])) - 0.2,
                                                          0))
                                                  })
                                                }),
                              mapping = ggplot2::aes(label = text, x = x, y = y,
                                                     color = treatment),
                              vjust = -0.5, hjust = 1, size = (alphaSZ/1.3) *
                                1.5, alpha = 0.75)
}
if (any(what %in% "meanSE")) {
  p <- p + ggplot2::stat_summary(ggplot2::aes(x = t.decay,
                                              y = value, color = treatment), fun = mean, geom = "line",
                                 linewidth = if (any(what %in% "models")) {
                                   0.35
                                 }
                                 else {
                                   1
                                 }, alpha = if (any(what %in% "models")) {
                                   0.6
                                 }
                                 else {
                                   1
                                 }) +
    ggplot2::stat_summary(ggplot2::aes(x = t.decay,
                                       y = value, color = treatment), fun.data = ggplot2::mean_se,
                          geom = "errorbar", linewidth = 0.35, alpha = 0.6)
}
if (any(what %in% "reps")) {
  p <- p + ggplot2::geom_point(ggplot2::aes(x = t.decay,
                                            y = value, color = treatment, shape = rep), size = 1.2,
                               alpha = 0.5)
}
if (any(what %in% "models")) {
  if (any(mod.results[geneID, (length(unique(DATA$treatment)) +
                               1):(length(unique(DATA$treatment)) * 2)] == 0)) {
    for (g in treatments) {
      p <- p + ggplot2::geom_line(data = data.frame(x = xlim[1]:xlim[2],
                                                    y = fun_exp(xlim[1]:xlim[2], a = unlist(mod.results[geneID,
                                                                                                        paste0("alpha_", gsub(" ", ".", g))]))),
                                  ggplot2::aes(x = x, y = y), color = colors[g],
                                  linewidth = 0.5, alpha = 1)
    }
  }
  else {
    for (g in treatments) {
      p <- p + ggplot2::geom_line(data = data.frame(x = xlim[1]:xlim[2],
                                                    y = dExp(xlim[1]:xlim[2], par = unlist(mod.results[geneID,
                                                                                                       paste0(c("alpha_", "beta_"), gsub(" ", ".",
                                                                                                                                         g))]))), ggplot2::aes(x = x, y = y), color = colors[g],
                                  linewidth = 0.5, alpha = 1)
    }
  }
}
p <- p + ggplot2::scale_color_manual("", breaks = names(colors),
                                     values = colors, labels = gsub("\\.", " ", treatments)) +
  ggplot2::coord_cartesian(xlim = xlim, ylim = ylim) +
  ggplot2::scale_y_continuous(breaks = yticks) + ggplot2::scale_x_continuous(breaks = xticks) +
  ggplot2::ylab("relative abundance") + ggplot2::xlab("time (min)") +
  ggplot2::scale_shape(guide = "none")
return(p)
}

#' contraint function list maker
#'
#' Individual double exponential models are all nested within model number 1 in
#' which alpha and beta parameters vary independently for each treatment. Models
#' that assume no difference in parameters between specific treatments manifest
#' as constraints in the modeling. These constraints are coded as functions
#' that are passed to the optimization process. Each model has a distinct
#' constraint function.
#'
#' @param mods data.frame specifying alpha and beta group pairs for each model
#' @param groups grouping matrix for alphas or betas
#'
#' @return  Returns a list of constraint functions to be passed to the
#' optimization function.
#'
#' @export
#'
#' @keywords internal
#'
#' @examples
#' constraint_fun_list_maker(mods = data.frame(a = c(1,1,1,2,2,2), b = c(1,2,3,1,2,3),
#'                           row.names = paste0('mod',1:6)),
#'                           groups = data.frame(treat1 = c(1,1,NA), treat2 = c(2,1,NA)))
#'

constraint_fun_list_maker <- function(mods, groups) {
  # function to make a function with an extra argument 'body.subset' to select lines
  # of the function body by numerical index
  make_function <-
    function(body.subset,
             args,
             body,
             last.line,
             env = parent.frame()) {
      f <- function() {

      }
      formals(f) <- args
      body(f) <- body[c(1:2, body.subset + 2, last.line - 1)]
      environment(f) <- env
      f
    }

  if (ncol(groups) == 4) {
    const_Ind = list(
      c(NULL),
      c(4, 5),
      c(2, 3),
      c(1, 3),
      c(1, 2),
      6,
      5,
      4,
      3,
      2,
      1,
      c(1, 6),
      c(2, 5),
      c(3, 4),
      c(1, 2, 3),
      NULL
    )
    names(const_Ind) <- paste0("gp", 1:16)
    const_Ind <- unlist(lapply(const_Ind[1:15], function(x)
      lapply(lapply(const_Ind,
                    function(y)
                      y + 6), function(z)
                        c(x, z))), recursive = FALSE)
    names(const_Ind) <- paste0("mod", 1:240)
    constraint_fun_list <- lapply(const_Ind, function(x) {
      make_function(
        body.subset = x,
        args = alist(pars =),
        body = as.call(c(
          as.name("{"),
          expression(
            g <-
              numeric(),
            g[length(g) + 1] <- pars[1] - pars[2],
            g[length(g) + 1] <- pars[1] - pars[3],
            g[length(g) + 1] <- pars[1] - pars[4],
            g[length(g) + 1] <- pars[2] - pars[3],
            g[length(g) + 1] <- pars[2] - pars[4],
            g[length(g) + 1] <- pars[3] - pars[4],
            g[length(g) + 1] <- pars[5] - pars[6],
            g[length(g) + 1] <- pars[5] - pars[7],
            g[length(g) + 1] <- pars[5] - pars[8],
            g[length(g) + 1] <- pars[6] - pars[7],
            g[length(g) + 1] <- pars[6] - pars[8],
            g[length(g) + 1] <- pars[7] - pars[8],
            return(g)
          )
        )),
        last.line = 16
      )
    })
  }

  if (ncol(groups) == 3) {
    const_Ind <- list(c(NULL), 3, 2, 1, 1:2, NULL)
    names(const_Ind) <- paste0("gp", 1:6)
    const_Ind <- unlist(lapply(const_Ind[1:5], function(x)
      lapply(lapply(const_Ind,
                    function(y)
                      y + 3), function(z)
                        c(x, z))), recursive = FALSE)
    names(const_Ind) <- paste0("mod", 1:30)
    constraint_fun_list <- lapply(const_Ind, function(x) {
      make_function(
        body.subset = x,
        args = alist(pars =),
        body = as.call(c(
          as.name("{"),
          expression(g <- numeric(),
            g[length(g) + 1] <- pars[1] - pars[2],
            g[length(g) + 1] <- pars[1] - pars[3],
            g[length(g) + 1] <- pars[2] - pars[3],
            g[length(g) + 1] <- pars[4] - pars[5],
            g[length(g) + 1] <- pars[4] - pars[6],
            g[length(g) + 1] <- pars[5] - pars[6],
            return(g)
          )
        )),
        last.line = 10
      )
    })
  }

  if (ncol(groups) == 2) {
    const_Ind <- list(c(NULL), 1, NULL)
    names(const_Ind) <- paste0("gp", 1:3)
    const_Ind <- unlist(lapply(const_Ind[1:2], function(x)
      lapply(lapply(const_Ind,
                    function(y)
                      y + 1), function(z)
                        c(x, z))), recursive = FALSE)
    names(const_Ind) <- paste0("mod", 1:6)
    constraint_fun_list <- lapply(const_Ind, function(x) {
      make_function(
        body.subset = x,
        args = alist(pars =),
        body = as.call(c(
          as.name("{"),
          expression(g <- numeric(),
                     g[length(g) + 1] <- pars[1] - pars[2],
                     g[length(g) + 1] <- pars[3] - pars[4],
                     return(g)
                     )
        )),
        last.line = 6
      )
    })


  }

  if (ncol(groups) == 1) {
    const_Ind <- list(NULL)
    names(const_Ind) <- paste0("gp", 1)
    const_Ind <- list(c(NULL), NULL)
    names(const_Ind) <- paste0("gp", 1)
    const_Ind <- unlist(lapply(const_Ind[1], function(x)
      lapply(lapply(const_Ind,
                    function(y)
                      y), function(z)
                        c(x, z))), recursive = FALSE)
    names(const_Ind) <- paste0("mod", 1:2)
    constraint_fun_list <- lapply(const_Ind, function(x) {
      make_function(
        body.subset = x,
        args = alist(pars =),
        body = as.call(c(
          as.name("{"),
          expression(g <- numeric(),
                     return(g)
                     )
        )),
        last.line = 4
      )
    })
  }
  return(constraint_fun_list)
}

#' number of Parameters function
#'
#' Calculates number of parameters for a specified model given the model parameter constraints.
#'
#' @param model model name (e.g. 'mod1')
#' @param mod two column data.frame with combination of alpha and beta grouping numbers in each model with rownames 'modX'
#' @param group matrix of all treatment alpha or beta equivalence groups
#'
#' @return returns the integer value of number of parameters in \code{model}
#'
#' @export
#'
#' @keywords internal
#'
#' @examples
#' n_par('mod1',data.frame('a'=1:5,'b'=rep(2,5),row.names=paste0('mod',1:5)),
#'          t(matrix(c(1,2,3,4,1,2,2,2,1,1,2,2,1,1,1,2,1,2,1,2,1,2,2,1),nrow=4)))


n_par <- function(model, mod, group) {
  length(unique(group[mod[model, "a"],])) + 1 + if (any(is.na(group[mod[model, "b"],]))) {
    0
  } else {
    length(unique(unlist(group[mod[model, "b"],])))
  }
}

#' calculates bounds for modeled parameters
#'
#' Calculates maximum and minimum bounds for parameter alpha based on experimental
#'   time points (t_0, t_1, t_2, t_3, ..., t_max). If RNA level is too low at t_1,
#'   then the decay has happened before our observations began - there is an upper
#'   bound to the decay rate we can detect (a_high).
#' If RNA level is too high at t_max, then relatively little decay has happened and
#'   we can not distinguish the decay rate and the decay of the decay rate  - there
#'   is a lower bound to the base decay rate of the decaying decay model (a_low).
#'
#' Similarly, limits on beta are required to prevent precude ranges in which the
#'   decay rate and decaing decay are indistinguishable. See vignette
#'   "RNAdecay_workflow" for more information.
#'
#' @param t_min time of first experiemtal time point after inhibition of
#'   transcription (not T0)
#' @param t_max time of last experimental time point
#'
#' @export
#'
#' @return returns the lowest/highest parameter values to be used as bounds on
#'   modeled parameters
#'
#' @examples
#' a_high(7.5)
#' a_low(480)
#' b_low(480)


a_high <- function(t_min) {
  signif(-(1 / t_min) * log(0.005), 2)
}

#' @rdname a_high
#' @export

a_low <- function(t_max) {
  10 ^ (floor(log(-(1 / t_max) * log(0.95), base = 10)))
}


#' @rdname a_high
#' @export

b_low <- function(t_max) {
  10 ^ (floor(log((1 / t_max), base = 10)))
}

## -----------------------------------------------------------------------------

# you can see that in this fabricated data the RNAs have a half-life of ~30 min
RPM_geneX <- data.frame(T0 = c(150, 135, 148), T30 = c(72, 76, 69), T60 = c(35, 35, 30), 
                     row.names = paste0("rep", 1:3))
RPM_geneX
RPM_geneX_T0norm <- RPM_geneX/mean(RPM_geneX$T0)
RPM_geneX_T0norm
colMeans(RPM_geneX_T0norm)

## -----------------------------------------------------------------------------
library(RNAdecay)
RPMs <- RNAdecay::RPMs # built-in example data
RPMs[1:2, c(1:11, 128)] # NOTE: not showing all columns here

## -----------------------------------------------------------------------------
cols(patterns = c("WT", "00"), df = RPMs) #gives the column indices that contain both tags in the column names

colnames(RPMs)[cols(patterns = c("WT", "00"), RPMs, "WT", "00")]

# NOTE: this is based on grep pattern matching so tags MUST be unique and non-nested (i.e. one entire label should not be part of another).

## -----------------------------------------------------------------------------
# create a directory for results output
wrdir <- paste0(tempdir(),"/DecayAnalysis/Example analysis results 1")
if(!file.exists(wrdir)) dir.create(wrdir, recursive = TRUE)

# specify sample tags from colnames of RPMs
treatment <- c("WT", "sov", "vcs", "vs") # NOTE: I did not use 'vcs sov' for the name of the double mutant, or cols(df=RPMs, patterns = c("vcs", "00")) would have indexed all T0 samples of both "vcs" and "vcs" sov"; instead, I used 'vs'.
reps <- c("r1", "r2", "r3", "r4")
tdecay <- c("00", "07", "15", "30", "60", "120", "240", "480") #again NO nesting of one label in another hence "00" instead of "0".

## -----------------------------------------------------------------------------
mean_RPM <- data.frame("geneID" = rownames(RPMs))
SE_RPM <- data.frame("geneID" = rownames(RPMs))
for(g in treatment){
  for(t in tdecay){
    mean_RPM <- cbind(mean_RPM, rowMeans(RPMs[, cols(df=RPMs, patterns = c(g, t))]))
    names(mean_RPM)[length(mean_RPM)] <- paste0(g, "_", t)
    SE_RPM <- cbind(SE_RPM, apply(X = RPMs[, cols(df=RPMs, patterns = c(g, t))], MARGIN = 1, FUN = stats::sd)/sqrt(length(reps)))
    names(SE_RPM)[length(SE_RPM)] <- paste0(g, "_", t)
  }}
mean_RPM[1:2, ]
SE_RPM[1:2, ]
# write output to file
write.table(x = mean_RPM, paste0(wrdir, "/RPM_mean.txt"), sep = "\t")
write.table(x = SE_RPM,   paste0(wrdir, "/RPM_SE.txt"), sep = "\t")

## -----------------------------------------------------------------------------
filt1 <- rep(TRUE, 118) # we will not filter in this example, so the filter value for each gene is set to TRUE.

## -----------------------------------------------------------------------------
mT0norm <- data.frame(row.names = rownames(RPMs)[filt1])
for(g in treatment){
  mean_T0reps <- rowMeans(RPMs[filt1, cols(df=RPMs, patterns=c(g, "00"))])
  for(r in reps){
    df <- RPMs[filt1, colnames(RPMs)[cols(df=RPMs, patterns=c(g, r))]]
    df <- df[, 1:length(tdecay)]/mean_T0reps
    mT0norm <- cbind(mT0norm, df)
  }}
write.table(x = mT0norm, file = paste0(wrdir, "/T0 normalized.txt"),  sep = "\t")


## -----------------------------------------------------------------------------
mean_mT0norm <- data.frame(row.names = rownames(mT0norm))
for(g in treatment){
  for(t in tdecay){
    mean_mT0norm <- cbind(mean_mT0norm, rowMeans(mT0norm[, cols(df=mT0norm, patterns=c(g, t))]))
    names(mean_mT0norm)[length(names(mean_mT0norm))] <- paste0(g, "_", t)
  }}

SE_mT0norm <- data.frame(row.names = rownames(mT0norm))
for(g in treatment){
  for(t in tdecay){
    SE_mT0norm <- cbind(SE_mT0norm, apply(X = mT0norm[, cols(df=mT0norm, patterns=c(g, t))], MARGIN = 1, FUN = function(x) stats::sd(x)/sqrt(length(reps))))
    names(SE_mT0norm)[length(names(SE_mT0norm))] <- paste0(g, "_", t)
  }}

# write output to file
write.table(x = mean_mT0norm, file = paste0(wrdir, "/T0 normalized_Mean.txt"),  sep = "\t")
write.table(x = SE_mT0norm, file = paste0(wrdir, "/T0 normalized_SE.txt"),  sep = "\t")

## -----------------------------------------------------------------------------
stablegenes <- c( "ATCG00490", "ATCG00680", "ATMG00280", "ATCG00580", "ATCG00140", "AT4G38970", "AT2G07671", "ATCG00570", "ATMG00730", "AT2G07727", "AT2G07687", "ATMG00160" ,"AT3G11630", "ATMG00060", "ATCG00600", "ATMG00220", "ATMG01170", "ATMG00410", "AT1G78900", "AT3G55440", "ATMG01320", "AT2G21170" ,"AT5G08670", "AT5G53300", "ATMG00070", "AT1G26630", "AT5G48300", "AT2G33040", "AT5G08690", "AT1G57720")

## -----------------------------------------------------------------------------
stabletable <- mean_mT0norm[stablegenes, ]
normFactors <- colMeans(stabletable)
write.table(x <- normFactors, paste0(wrdir, "/Normalziation Decay Factors.txt"), sep = "\t")
normFactors_mean <- matrix(normFactors, nrow = length(tdecay))
normFactors_SE <- matrix(apply(X = stabletable, MARGIN = 2, function(x) stats::sd(x)/sqrt(length(stablegenes))), nrow = length(tdecay))
t.decay <- c(0, 7.5, 15, 30, 60, 120, 240, 480)
rownames(normFactors_mean) <- t.decay
rownames(normFactors_SE) <- t.decay
colnames(normFactors_mean) <- treatment
colnames(normFactors_SE) <- treatment
list(normalizationFactors = normFactors_mean, SE = normFactors_SE)

## -----------------------------------------------------------------------------
nF <- vector()
ind <- sapply(names(normFactors), function(x) grep(x, colnames(mT0norm)))
for(i in 1:ncol(ind)){
nF[ind[, i]] <- colnames(ind)[i]
}
normFactorsM <- t(matrix(rep(normFactors[nF], nrow(mT0norm)), ncol = nrow(mT0norm)))
rm(nF, ind)

## -----------------------------------------------------------------------------
mT0norm_2 <- data.frame(mT0norm/normFactorsM, 
                     row.names = rownames(mT0norm))

write.table(mT0norm_2, paste0(wrdir, "/T0 normalized and decay factor corrected.txt"), sep = "\t")

## -----------------------------------------------------------------------------
mT0norm_2.1 <- reshape2::melt(as.matrix(mT0norm_2), varnames = c("geneID", "variable"))
mT0norm_2.1 <- cbind(mT0norm_2.1, reshape2::colsplit(mT0norm_2.1$variable, "_", names = c("treatment", "t.decay", "rep")))

mT0norm_2.1 <- mT0norm_2.1[, colnames(mT0norm_2.1) !=  "variable"]
mT0norm_2.1 <- mT0norm_2.1[, c(1, 3, 4, 5, 2)]
colnames(mT0norm_2.1) <- c("geneID", "treatment", "t.decay", "rep", "value")
mT0norm_2.1$rep <- gsub("r", "rep", mT0norm_2.1$rep)
mT0norm_2.1$t.decay <- as.numeric(gsub("7", "7.5", as.numeric(mT0norm_2.1$t.decay)))
mT0norm_2.1$treatment <- factor(mT0norm_2.1$treatment,levels = c("WT","sov","vcs","vs"))
mT0norm_2.1$rep <- factor(mT0norm_2.1$rep, levels = paste0("rep",1:4))
write.table(x = mT0norm_2.1,  file = paste0(wrdir, 
"/ExampleDecayData+stableGenes.txt"), sep = "\t")

## -----------------------------------------------------------------------------
table(RNAdecay::decay_data[,1:4] == mT0norm_2.1[,1:4]) # should be all TRUE no FALSE

## -----------------------------------------------------------------------------
library(RNAdecay)
decay_data <- RNAdecay::decay_data # built-in example data

# make new directory for results
wrdir <- paste0(tempdir(),"/DecayAnalysis/Example analysis results 2")
if(!file.exists(wrdir)) dir.create(wrdir, recursive = TRUE)

## -----------------------------------------------------------------------------
levels(decay_data$treatment) # This can not be longer than 4 elements for modeling.

# reorder factor levels if necessary (skipped here):
# decay_data$treatment <- factor(decay_data$treatment, levels = levels(decay_data$treatment)[c(4, 1, 2, 3)]) # numbers here refer to the current order position in the factor levels() - the order of the numbers designates the new position

levels(decay_data$treatment)[4] <- "vcs.sov"
levels(decay_data$treatment)

## -----------------------------------------------------------------------------
decay_data <- decay_data[order(decay_data$t.decay), ]
decay_data <- decay_data[order(decay_data$rep), ]
decay_data <- decay_data[order(as.numeric(decay_data$treatment)), ] 
decay_data <- decay_data[order(decay_data$geneID), ]
ids <- as.character(unique(decay_data$geneID)) # 118 in example set
decay_data[1:10, ]

## -----------------------------------------------------------------------------
# (no changes needed here - just execute the code)
nEquivGrp <- if (length(unique(decay_data$treatment)) == 2) {2} else
  if (length(unique(decay_data$treatment)) == 3) {5} else 
    if (length(unique(decay_data$treatment)) == 4) {15}
genoSet <- 1:(length(unique(decay_data$rep)) * length(unique(decay_data$t.decay)))
nTreat <- length(unique(decay_data$treatment))
nSet <- length(genoSet)*nTreat
groups <- groupings(decay_data)
mods <- data.frame(
  "a" = as.vector(sapply(1:nEquivGrp, function(x) {rep(x, nEquivGrp + 1)})),
  "b" = rep(1:(nEquivGrp + 1), nEquivGrp),
  row.names = paste0("mod", 1:(nEquivGrp*(nEquivGrp+1)))
)

## -----------------------------------------------------------------------------
group_map(decaydata = decay_data, path = paste0(wrdir, "/Model grouping colormap.pdf"), nEquivGrp = nEquivGrp, groups = groups, mods = mods)

## -----------------------------------------------------------------------------
a_bounds <- c(a_low(max(decay_data$t.decay)),
             a_high(min(unique(decay_data$t.decay)[unique(decay_data$t.decay)>0])))
b_bounds <- c(b_low(max(decay_data$t.decay)), 0.075)
a_bounds;b_bounds

## -----------------------------------------------------------------------------
mod_optimization("AT2G18150", decay_data, group = groups, mod = mods, a_bounds, b_bounds, c("mod1", "mod2", "mod16", "mod239", "mod240"), file_only = FALSE, path = wrdir) 
mod_optimization("AT4G09680", decay_data, group = groups, mod = mods, a_bounds, b_bounds, c("mod1", "mod2", "mod16", "mod239", "mod240"), file_only = FALSE, path = wrdir) 

## -----------------------------------------------------------------------------
####### To run all genes in parallel use:
# parallel::mclapply(ids, FUN = mod_optimization, 
#                    data = decay_data, group = groups, mod = mods, 
#                    alpha_bounds = a_bounds, beta_bounds = b_bounds,
#                    models = paste0("mod", 1:240), 
#                    path = paste0(wrdir, "/modeling_results"),
#   mc.cores = getOption("mc.cores",  15L), # set the number of compute cores to use here (e.g., 9L = 9 cores, 11L = 11 cores)
#   mc.preschedule = TRUE,
#   mc.set.seed = TRUE,
#   mc.silent = FALSE,
#   mc.cleanup = TRUE,
#   mc.allow.recursive = TRUE)

## -----------------------------------------------------------------------------
test_ids <- sample(ids, 1) # NOTE: that everytime this line is run it generates a different random sampling, therefore the gene modeled below will be different each time this code is run. To test the exact gene shown in the vignette make a new character vector of the gene id reported below and pass it to the gene argument of mod_optimization using lapply instead of passing 'test_ids' as we do here.  
test_ids

a <- proc.time()[3]
models <- lapply(X = test_ids, # alternatively use `ids` here to complete the entire sample data set, but be prepared to wait ~ 10 h. These gene IDs will get passed one at time to the "gene" argument of mod_optimization() and return a list of the results data frame.
                FUN = mod_optimization,
                data = decay_data, 
                group = groups, 
                mod = mods,
                alpha_bounds = a_bounds, 
                beta_bounds = b_bounds,
                models = rownames(mods), 
                path = paste0(wrdir, "/modeling_results"),
                file_only = FALSE) 
names(models) <- test_ids
b <- proc.time()[3]
(b-a)/60/length(test_ids) # gives you average min per gene 

## -----------------------------------------------------------------------------
models <- lapply( paste0( wrdir, "/modeling_results/", test_ids, "_results.txt"), read.delim, header = TRUE )   
names(models) <- test_ids

## -----------------------------------------------------------------------------
models <- RNAdecay::models # built-in example data, comment this line out to continue with your own modelling output
results <- t(sapply(models, function(x) x[x[, "AICc"] == min(x[, "AICc"]), ]))
results <- as.data.frame(results)
results[, 1:2] <- sapply(as.data.frame(results[, 1:2]), function(x) as.character(unlist(x)))
results[, -c(1,2)] <- sapply(results[, -c(1,2)], unlist)
write.table(results, file = paste0(wrdir,"/best model results.txt"), sep = "\t")
results <- read.delim(paste0(wrdir,"/best model results.txt"))

## -----------------------------------------------------------------------------
library(ggplot2)
pdf(paste0(wrdir,"/distributions of stats.pdf"))
p <- ggplot(results)
print(p+geom_histogram(aes(x = sigma2), bins = 300)+
        coord_cartesian(xlim = c(0,0.5))+
        geom_vline(xintercept = 0.0625,color = "red2")+
        plain_theme())
print(p+stat_bin(aes(x = sigma2), breaks = c(seq(0,0.25,0.25/50),1), geom = "bar")+coord_cartesian(xlim = c(0,0.25)))
print(p+stat_ecdf(aes(sigma2), geom = "line")+
        coord_cartesian(xlim = c(0,0.5)))
print(p+stat_ecdf(aes(sqrt(sigma2)), geom = "line")+
        coord_cartesian(xlim = c(0,sqrt(0.5))))
print(p+geom_histogram(aes(x = range.LL), bins = 60))
print(p+geom_histogram(aes(x = nUnique.LL), bins = 60))
dev.off()

pdf(paste0(wrdir,"/lowest AICc model counts.pdf"), height = 8, width = 32)
p <- ggplot(data = data.frame(
  model = as.integer(gsub("mod","",names(table(results$mod)))),
  counts = as.integer(table(results$mod))))+
  geom_bar(aes(x = model, y = counts), stat = "identity")+
  scale_x_continuous(limits = c(0,nrow(mods)), breaks = seq(0,nrow(mods),5))+
  ggtitle("model distribution of absolute lowest AICs")
print(p+plain_theme(25))
dev.off()

## -----------------------------------------------------------------------------
min_mods <- sapply(models, function(x) which (x[, "AICc"] < (2+min(x[, "AICc"])))) 
min_alpha_mods <- lapply(min_mods, function(x) unique(mods[x, "a"]))

pdf(paste0(wrdir,"/number of models that performed similar to the one selected.pdf"))
barplot(height = table(sapply(min_mods, length)), xlab = "No. models in the lowest AICc group (not more than 2 different from lowest)",
        ylab = "No. genes")
barplot(height = table(sapply(min_alpha_mods, length)), xlab = "No. alpha groups in the lowest AICc group (not more than 2 different from lowest)",
        ylab = "No. genes")
dev.off()

## -----------------------------------------------------------------------------
results <- read.delim(paste0(wrdir,"/best model results.txt"))
results$alpha_grp <- mods[as.character(results$mod), "a"]
results$beta_grp <- mods[as.character(results$mod), "b"]
results$mod <- as.numeric(gsub("mod", "", as.character(results$mod)))

results$alphaPattern <- sapply(rownames(results), function(x) {
  paste0(gsub("alpha_", "", colnames(results)[3:(2+nTreat)][order(unlist(round(results[x, 3:(2+nTreat)], 4)))]), collapse = "<=")
  })
results$alphaPattern <- paste0(results$alpha_grp, "_", results$alphaPattern)
results$betaPattern <- sapply(rownames(results), function(x){
  paste0(gsub("beta_", "", colnames(results)[(3+nTreat):(2+2*nTreat)][order(unlist(round(results[x, (3+nTreat):(2+2*nTreat)], 4)))]), collapse = "<=")
  })
results$betaPattern <- paste0(results$beta_grp, "_", results$betaPattern)

results <- results[order(rownames(results)), ]
results <- results[order(results$beta_grp), ]
results <- results[order(results$alphaPattern), ]
results <- results[order(results$alpha_grp), ]

results$alphaPattern <- factor(results$alphaPattern, levels = as.character(unique(results$alphaPattern)))

results <- data.frame(results[, 3:(2*nTreat+3), 2], results[, c("AICc", "alpha_grp", "beta_grp", "alphaPattern", "betaPattern")])
results$nEqMods <- sapply(min_mods[rownames(results)], length)
results$nEqAgp <- sapply(min_alpha_mods[rownames(results)], length)

# Customize: add columns of relative alphas and betas as desired, e.g.:
results$rA_sov.WT    <- results$alpha_sov      / results$alpha_WT
results$rA_vcs.WT    <- results$alpha_vcs      / results$alpha_WT
results$rA_vcssov.WT <- results$alpha_vcs.sov  / results$alpha_WT

write.table(results, paste0(wrdir,"/alphas+betas+mods+grps+patterns+relABs.txt"), sep = "\t")

results <- read.delim(paste0(wrdir,"/alphas+betas+mods+grps+patterns+relABs.txt"), header = TRUE, colClasses =  c(NA, rep("numeric", 2+2*nTreat), rep("integer", 2), rep("character", 2), rep("integer", 2), rep("numeric", 3)))
# results$alpha_subgroup <- factor(results$alpha_subgroup, levels = unique(results$alpha_subgroup))
results$alphaPattern <- factor(results$alphaPattern, levels = unique(results$alphaPattern))
results$betaPattern <- factor(results$betaPattern, levels = unique(results$betaPattern))
results[1:3, ]

## -----------------------------------------------------------------------------
library(RNAdecay)
decay_data <- RNAdecay::decay_data # built-in package example data; comment this line out to use your own decay_data
decay_data$treatment <- factor(decay_data$treatment, levels = c("WT", "sov", "vcs", "vs")) # you must type them identically in the new order  
levels(decay_data$treatment)[4] <- "vcs.sov"
decay_data[1:4,]

## -----------------------------------------------------------------------------
results <- RNAdecay::results # built-in package example data; comment this line out to use your own results
# For example:
# results <- read.delim(paste0(tempdir(),"/DecayAnalysis/Example analysis results 2/alphas+betas+mods+grps+patterns+relABs.txt"), header = TRUE, colClasses =  c(NA, rep("numeric", 9), rep("integer", 3), rep("character", 3), rep("numeric", 6)))
# results$alpha_subgroup <- factor(results$alpha_subgroup, levels = unique(results$alpha_subgroup))
results[1:3, ]

## -----------------------------------------------------------------------------
# gene_desc <- read.delim("gene_description_20140101.txt"), quote = NULL, comment = '', header = FALSE)
# gene_desc[, 1] <- substr(gene_desc[, 1], 1, 9)
# gene_desc <- data.frame(gene_desc[!duplicated(gene_desc[, 1]), ], row.names = 1)
# colnames(gene_desc) <- c("type", "short description", "long description", "computational description")
# descriptions <- gene_desc[, "long description"]
# names(descriptions) <- rownames(gene_desc)

descriptions <- c(
  "Encodes a ubiquitin E3 ligase with RING and SPX domains that is involved in mediating immune responses and mediates degradation of PHT1s at plasma membranes.  Targeted by MIR827. Ubiquitinates PHT1;3, PHT1;2, PHT1;1/AtPT1 and PHT1;4/AtPT2.",
  "",
  "Related to Cys2/His2-type zinc-finger proteins found in higher plants. Compensated for a subset of calcineurin deficiency in yeast. Salt tolerance produced by ZAT10 appeared to be partially dependent on ENA1/PMR2, a P-type ATPase required for Li+ and Na+ efflux in yeast. The protein is localized to the nucleus, acts as a transcriptional repressor and is responsive to chitin oligomers. Also involved in response to photooxidative stress.",
  "Encodes a stress enhanced protein that localizes to the thylakoid membrane and whose mRNA is upregulated in response to high light intensity.  It may be involved in chlorophyll binding."
  )
names(descriptions) <- c("AT1G02860","AT5G54730", "AT1G27730", "AT4G34190")

## ----fig.show = 'hold'--------------------------------------------------------
p <- decay_plot(geneID = "AT1G02860", 
              xlim = c(0, 350), 
              ylim = c(0, 1.25), 
              xticks = 1:5*100, 
              yticks = 0:5/4,
              alphaSZ  =  12, 
              what  =  c("meanSE", "reps", "models"),
              treatments  =  c("WT", "vcs.sov"), 
              colors = c("darkblue", "red3"),
              DATA = decay_data, 
              mod.results = results, 
              gdesc = descriptions)
# print(p+plain_theme(8, 1, leg.pos = c(0.7, 0.8))) #this will print the plot to your current graphics device (dev.cur() tells you what that is), if you do not have a graphics device open (e.g., "null device") initiate one (e.g., use quartz(),pdf(), or windows(); dev.off() closes the device and writes the file).

## -----------------------------------------------------------------------------
# make new directory for results
wrdir <- paste0(tempdir(),"/DecayAnalysis/Example analysis results 3")
if(!file.exists(wrdir)) dir.create(wrdir, recursive = TRUE)

pdf(paste0(wrdir, "/decay plot example.pdf"), width = 10, height = 8)
p <- decay_plot(geneID = "AT1G02860", 
              xlim = c(0, 500), 
              ylim = c(0, 1.25), 
              xticks = 1:5*100, 
              yticks = 0:5/4,
              alphaSZ = 10, 
              what = c("Desc","meanSE", "reps", "models","alphas&betas"),
              treatments = c("WT", "vcs.sov"),
              colors = c("darkblue", "red3"),
              DATA = decay_data,
              mod.results = results,
              gdesc = descriptions)
print(p+plain_theme(32, 1, leg.pos = 'right'))
dev.off()

# plot multiple genes
ids <- c("AT5G54730", "AT1G27730", "AT4G34190")
# or e.g.,
# ids <- rownames(results[results$alpha_WT > 0.1, ]) 

pdf(paste0(wrdir, "/multiple RNA decay plots.pdf"), width = 10, height = 7)
for(i in ids){
  p <- decay_plot(i, 
                what = c("meanSE", "models", "Desc"),
                mod.results = results, 
                gdesc = descriptions,
                treatments = c("WT","sov","vcs","vcs.sov"),
                DATA = decay_data)
  print(p+plain_theme(13.8, 1, leg.pos = 'right'))
  cat(i, " plotted.\n")
}
dev.off()

## -----------------------------------------------------------------------------

pdf(paste0(wrdir, "/halflife plot example.pdf"), width = 10, height = 8)

p <- hl_plot(geneID = "AT1G02860",
             gene_symbol = "SYG1",
             df_decay_rates = RNAdecay::results,
             hl_dist_treatment = "WT",
             hl_treatment = c("WT","sov","vcs","vcs.sov"),
             arrow_lab_loc = "key"
             )
  
print(p)
dev.off()

# plot multiple genes
ids <- c("AT5G54730", "AT1G27730", "AT4G34190")
# or e.g.,
# ids <- rownames(results[results$alpha_WT > 0.1, ]) 

pdf(paste0(wrdir, "/multiple halflife plots.pdf"), width = 4, height = 4)
for(i in ids) {
  
p <- hl_plot(geneID = i,
             df_decay_rates = RNAdecay::results,
             hl_dist_treatment = "WT",
             hl_treatment = c("WT","sov","vcs","vcs.sov"),
             arrow_lab_loc = "plot"
             )
  print(p+plain_theme(8, 1,leg.pos = 'right'))
  cat(i, " plotted.\n")

  }
dev.off()

## -----------------------------------------------------------------------------
sessionInfo()

#' Normalized RNA abundance RNA decay timecourse
#'
#' A long form dataset of RNA abundance of 118 genes in four Arabidopsis thaliana genotypes (WT, sov, vcs, vcs sov). Four biological replicates were collectred 0, 7.5, 15, 30, 60, 120, 240, 480 min after blocking transcription. RNA was extracted, subjected to ribodepletion, and sequenced by RNA-seq (Illumina 50 nt single end reads). RPM values were normalized to mean T0 abundance and corrected by a decay factor.
#'
#' @format a data frame with 5 columns and 15104 rows.
#'  \describe{
#'     \item{geneID}{gene identifier; AGI}
#'     \item{treatment}{Arabidopsis genotype}
#'     \item{t.decay}{time of decay, in minutes}
#'     \item{rep}{replicate number}
#'     \item{value}{RPM value normalized to the replicate samples' mean T0 abundance and decay factor corrected}
#'     }
#'
#' @source Sorenson et al. (2017) Submitted; \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE86361}
#'
"decay_data"

---
title: "_RNAdecay_ Vignette: Normalization, Modeling, and Visualization of RNA Decay Data"
author: "Reed Sorenson | reed.sorenson@utah.edu | Department of Biology | University of Utah"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{RNAdecay Vignette: Normalization, Modeling, and Visualization of RNA Decay Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# I. DATA NORMALIZATION AND CORRECTION

The functions and code of the `RNAdecay` package were written for the purpose of analyzing decreasing levels of RNA abundance as measured by RNA short read sequencing (RNA-seq) after inhibition of transcription. Tissue is treated to block transcription (e.g., using a chemical such as cordycepin or ActinomycinD) and collected at regular intervals (referred to as the RNA decay time course, e.g., T0, T30, T60 refer to 0, 30, and 60 min of blocked transcription). Total RNA is extracted from the tissue and measured by RNA-seq. Although we wrote this package to analyze RNA-seq data, it could also be adapted to analyze individual gene data from quantitative reverse transcription polymerase chain reaction (qRT-PCR) measures, however, normalization (as herein described) is best performed using a number of stable reference genes. 

Raw (Illumina) RNA-seq data are typically libraries made of up 10M-250M short (50 nt) sequences. These sequences are aligned to the genome and counted based on the location of their alignment in annotated genomic ranges (e.g., genes, exons, etc). Read counts for each gene are normalized to the size of their respective RNA-seq library for comparison to libraries generated from other biological material, therefore, read counts are expressed as reads per million (RPM).  We begin RNA decay analysis in this package with RPM RNA abundance data. 

## 1. T0 NORMALIZATION
Prior to modeling RNA decay, we normalize the read counts to the initial (T0) RNA abundance, the transcript level just prior to blocking transcription. This is unique for each gene, and, in an experiment with multiple replicates (reps), we use the mean value for all T0 reps. All RPM values in the time course are divided by the mean T0 value. For example:

```{r}

# you can see that in this fabricated data the RNAs have a half-life of ~30 min
RPM_geneX <- data.frame(T0 = c(150, 135, 148), T30 = c(72, 76, 69), T60 = c(35, 35, 30), 
                     row.names = paste0("rep", 1:3))
RPM_geneX
RPM_geneX_T0norm <- RPM_geneX/mean(RPM_geneX$T0)
RPM_geneX_T0norm
colMeans(RPM_geneX_T0norm)
```

_The following workflow depends on beginning with a data frame of RPM values with column names consisting of unique, non-nested variable tags separated by underscores ('\_'), with rownames as unique gene identifiers._ Here, we use with built-in example data (from [Sorenson et al., 2018](http://www.pnas.org/content/early/2018/01/30/1712312115.short)) which we will use throughout this workflow. It has 4 genotypes, 8 time points, and 4 biological replicates for 128 samples (columns) and 118 genes (rows). 
```{r}
library(RNAdecay)
RPMs <- RNAdecay::RPMs # built-in example data
RPMs[1:2, c(1:11, 128)] # NOTE: not showing all columns here
```

Biological replicate data columns are indexed using the `cols()` function.
```{r}
cols(patterns = c("WT", "00"), df = RPMs) #gives the column indices that contain both tags in the column names

colnames(RPMs)[cols(patterns = c("WT", "00"), RPMs, "WT", "00")]

# NOTE: this is based on grep pattern matching so tags MUST be unique and non-nested (i.e. one entire label should not be part of another).
```

The mean and standard error (SE) RPM values are calculated by looping over label combinations and binding the values together in a new data frame.
```{r}
# create a directory for results output
wrdir <- paste0(tempdir(),"/DecayAnalysis/Example analysis results 1")
if(!file.exists(wrdir)) dir.create(wrdir, recursive = TRUE)

# specify sample tags from colnames of RPMs
treatment <- c("WT", "sov", "vcs", "vs") # NOTE: I did not use 'vcs sov' for the name of the double mutant, or cols(df=RPMs, patterns = c("vcs", "00")) would have indexed all T0 samples of both "vcs" and "vcs" sov"; instead, I used 'vs'.
reps <- c("r1", "r2", "r3", "r4")
tdecay <- c("00", "07", "15", "30", "60", "120", "240", "480") #again NO nesting of one label in another hence "00" instead of "0".
```

Loop over the treatments and timepoints to subset the appropriate columns, and calculate mean and SE of replicates.
```{r}
mean_RPM <- data.frame("geneID" = rownames(RPMs))
SE_RPM <- data.frame("geneID" = rownames(RPMs))
for(g in treatment){
  for(t in tdecay){
    mean_RPM <- cbind(mean_RPM, rowMeans(RPMs[, cols(df=RPMs, patterns = c(g, t))]))
    names(mean_RPM)[length(mean_RPM)] <- paste0(g, "_", t)
    SE_RPM <- cbind(SE_RPM, apply(X = RPMs[, cols(df=RPMs, patterns = c(g, t))], MARGIN = 1, FUN = stats::sd)/sqrt(length(reps)))
    names(SE_RPM)[length(SE_RPM)] <- paste0(g, "_", t)
  }}
mean_RPM[1:2, ]
SE_RPM[1:2, ]
# write output to file
write.table(x = mean_RPM, paste0(wrdir, "/RPM_mean.txt"), sep = "\t")
write.table(x = SE_RPM,   paste0(wrdir, "/RPM_SE.txt"), sep = "\t")
```

Optionally, an RPM-based filtering step could be applied at this point to remove lowly expressed genes, for example.
```{r}
filt1 <- rep(TRUE, 118) # we will not filter in this example, so the filter value for each gene is set to TRUE.
```

Replicate mean T0 RPM values are then used to normalize gene level data.
```{r}
mT0norm <- data.frame(row.names = rownames(RPMs)[filt1])
for(g in treatment){
  mean_T0reps <- rowMeans(RPMs[filt1, cols(df=RPMs, patterns=c(g, "00"))])
  for(r in reps){
    df <- RPMs[filt1, colnames(RPMs)[cols(df=RPMs, patterns=c(g, r))]]
    df <- df[, 1:length(tdecay)]/mean_T0reps
    mT0norm <- cbind(mT0norm, df)
  }}
write.table(x = mT0norm, file = paste0(wrdir, "/T0 normalized.txt"),  sep = "\t")

```

The mean and standard error of the T0 normalized data are then calculated for replicate samples.
```{r}
mean_mT0norm <- data.frame(row.names = rownames(mT0norm))
for(g in treatment){
  for(t in tdecay){
    mean_mT0norm <- cbind(mean_mT0norm, rowMeans(mT0norm[, cols(df=mT0norm, patterns=c(g, t))]))
    names(mean_mT0norm)[length(names(mean_mT0norm))] <- paste0(g, "_", t)
  }}

SE_mT0norm <- data.frame(row.names = rownames(mT0norm))
for(g in treatment){
  for(t in tdecay){
    SE_mT0norm <- cbind(SE_mT0norm, apply(X = mT0norm[, cols(df=mT0norm, patterns=c(g, t))], MARGIN = 1, FUN = function(x) stats::sd(x)/sqrt(length(reps))))
    names(SE_mT0norm)[length(names(SE_mT0norm))] <- paste0(g, "_", t)
  }}

# write output to file
write.table(x = mean_mT0norm, file = paste0(wrdir, "/T0 normalized_Mean.txt"),  sep = "\t")
write.table(x = SE_mT0norm, file = paste0(wrdir, "/T0 normalized_SE.txt"),  sep = "\t")
```



## 2. DECAY FACTOR CORRECTION

After inhibition of RNA synthesis, RNA degradation continues causing the total pool of RNA in cells to decrease. As this occurs, very stable RNAs become a much larger proportion of the total RNA pool even though their concentration in the cell remains nearly constant. RNA-seq RPM data is scaled to a normalized library size (i.e. to the total RNA pool), and, because of this, abundance of stable RNAs appears to increase. The apparent increase in abundance of stable genes is proportional to the reduction in the total pool of RNA, therefore, we can use it to apply a correction to the data. We call this decay factor correction. The decay factor is estimated based on the RPM increase (relative to the T0 samples) of a set of stable and abundant reference genes.

We selected 30 genes with high abundance on which to calculate the decay factors for each sample. These were then used to correct abundance in each of the respective samples. A unique decay factor is calculated for each treatment at each time point.

First, make a vector of 'geneID's of abundant and stable reference genes present in the data set. 
```{r}
stablegenes <- c( "ATCG00490", "ATCG00680", "ATMG00280", "ATCG00580", "ATCG00140", "AT4G38970", "AT2G07671", "ATCG00570", "ATMG00730", "AT2G07727", "AT2G07687", "ATMG00160" ,"AT3G11630", "ATMG00060", "ATCG00600", "ATMG00220", "ATMG01170", "ATMG00410", "AT1G78900", "AT3G55440", "ATMG01320", "AT2G21170" ,"AT5G08670", "AT5G53300", "ATMG00070", "AT1G26630", "AT5G48300", "AT2G33040", "AT5G08690", "AT1G57720")
```

Mean T0 normalized values of stable genes are then pulled out of the `mean_mT0norm` data frame and used to calculate the decay factor for each set of replicate samples.
```{r}
stabletable <- mean_mT0norm[stablegenes, ]
normFactors <- colMeans(stabletable)
write.table(x <- normFactors, paste0(wrdir, "/Normalziation Decay Factors.txt"), sep = "\t")
normFactors_mean <- matrix(normFactors, nrow = length(tdecay))
normFactors_SE <- matrix(apply(X = stabletable, MARGIN = 2, function(x) stats::sd(x)/sqrt(length(stablegenes))), nrow = length(tdecay))
t.decay <- c(0, 7.5, 15, 30, 60, 120, 240, 480)
rownames(normFactors_mean) <- t.decay
rownames(normFactors_SE) <- t.decay
colnames(normFactors_mean) <- treatment
colnames(normFactors_SE) <- treatment
list(normalizationFactors = normFactors_mean, SE = normFactors_SE)
```

Generate a matrix of the same dimensions as `mT0norm` with the appropriate decay factors in the same respective positions.
```{r}
nF <- vector()
ind <- sapply(names(normFactors), function(x) grep(x, colnames(mT0norm)))
for(i in 1:ncol(ind)){
nF[ind[, i]] <- colnames(ind)[i]
}
normFactorsM <- t(matrix(rep(normFactors[nF], nrow(mT0norm)), ncol = nrow(mT0norm)))
rm(nF, ind)
```

Apply the decay factor corrections.
```{r}
mT0norm_2 <- data.frame(mT0norm/normFactorsM, 
                     row.names = rownames(mT0norm))

write.table(mT0norm_2, paste0(wrdir, "/T0 normalized and decay factor corrected.txt"), sep = "\t")
```

Rearrange the RNA decay data into a long form data frame for modeling and visualization.
```{r}
mT0norm_2.1 <- reshape2::melt(as.matrix(mT0norm_2), varnames = c("geneID", "variable"))
mT0norm_2.1 <- cbind(mT0norm_2.1, reshape2::colsplit(mT0norm_2.1$variable, "_", names = c("treatment", "t.decay", "rep")))

mT0norm_2.1 <- mT0norm_2.1[, colnames(mT0norm_2.1) !=  "variable"]
mT0norm_2.1 <- mT0norm_2.1[, c(1, 3, 4, 5, 2)]
colnames(mT0norm_2.1) <- c("geneID", "treatment", "t.decay", "rep", "value")
mT0norm_2.1$rep <- gsub("r", "rep", mT0norm_2.1$rep)
mT0norm_2.1$t.decay <- as.numeric(gsub("7", "7.5", as.numeric(mT0norm_2.1$t.decay)))
mT0norm_2.1$treatment <- factor(mT0norm_2.1$treatment,levels = c("WT","sov","vcs","vs"))
mT0norm_2.1$rep <- factor(mT0norm_2.1$rep, levels = paste0("rep",1:4))
write.table(x = mT0norm_2.1,  file = paste0(wrdir, 
"/ExampleDecayData+stableGenes.txt"), sep = "\t")
```

Following the steps in this vignette, the resulting normalized data frame should be identical to the one supplied in this package called decay_data. To check:
```{r}
table(RNAdecay::decay_data[,1:4] == mT0norm_2.1[,1:4]) # should be all TRUE no FALSE
```

The normalized data is now ready to be used for modeling decay rates. 



# II. MODELING DECAY RATES

RNA degradation is typically modeled with a constant exponential decay model. This model assumes a constant decay rate throughout the decay time course. However, a number of issues might violate this assumption. For example, decay might depend on continuous transcription of rapidly turned over components of decay machinery; inhibition of this transcription would cause slowed decay due to lost supply of decay components. Alternatively, decay rate might be regulated (e.g., diurnally) leading to a slow change in decay rate over a long decay time course. Feedback due to slowed transcription could also lead to slowed RNA decay. We, therefore, apply both a constant exponential decay model and a time-dependent exponential decay model. However, besides these possibilies, other distinct mechanisms could lead to an apparent slowing of RNA decay. For example, because RNA-seq reads are pooled from multiple cell types and mapped to genes, distinct gene mRNA subpopulations might have different decay rates (i.e., different splice isoforms, or the same mRNA in different cell types of a multicellular organism) that are are averaged when these are pooled together.  We believe that for these the time-dependent exponential decay model will also better capture this average than a constant exponential decay model. 

The dynamic nature of RNA decay can be a point of gene regulation. To identify treatments that might affect decay rate, we model all possibilities of treatment effect on both [initital] decay rates ($\alpha$) and decay of decay rates ($\beta$) using maximum likelihood modeling. This is done by running the modeling with constraints on treatment decay rates (i.e., constraining decay rates of treatments to be equal or allowing them to combinatorially vary in independent groups). This can be done with up to four treatments in this package. Modeling is performed one gene at a time with each set of constraints, and then each set of parameter estimates (from each set of contraints) are compared using the AICc statistic. In this package, we refer to these as equivalence constraint groupings. 

For the detailed mathematical framework of the decaying decay model see [Sorenson et. al. (2018)](http://www.pnas.org/content/early/2018/01/30/1712312115.short). Below we describe the steps of the algorithmic implementation and walk though an example data set. 


## 3. LOAD NORMALIZED AND CORRECTED DATA AND CHECK THE FORMAT

Load normalized and corrected data and check the format: the following script requires a data frame with five columns with the following column names exactly: "geneID", "treatment", "t.decay", "rep", "value":

    * geneID    = gene identifiers (`factor`)
    * treatment = experimental treatment or genotype (`factor`)
    * t.decay   = time after transcriptional inhibition (`numeric`)
    * rep       = biological replicate number (`factor`)
    * value     = RNA abundance normalized to library size, decay factor and mean T0 values (`numeric`)
    
```{r}
library(RNAdecay)
decay_data <- RNAdecay::decay_data # built-in example data

# make new directory for results
wrdir <- paste0(tempdir(),"/DecayAnalysis/Example analysis results 2")
if(!file.exists(wrdir)) dir.create(wrdir, recursive = TRUE)
```

Rename factor levels and specify level order of `decay_data$treatment` for sorting and making figures. The example data set is already ordered, but when data is read from a text file (e.g., .csv) factor levels are ordered automatically and should be reordered according to user preference now to avoid later headaches.
```{r}
levels(decay_data$treatment) # This can not be longer than 4 elements for modeling.

# reorder factor levels if necessary (skipped here):
# decay_data$treatment <- factor(decay_data$treatment, levels = levels(decay_data$treatment)[c(4, 1, 2, 3)]) # numbers here refer to the current order position in the factor levels() - the order of the numbers designates the new position

levels(decay_data$treatment)[4] <- "vcs.sov"
levels(decay_data$treatment)
```

Sort the `decay_data` data frame as follows (*NOTE: REQUIRED step for proper indexing below*).
```{r}
decay_data <- decay_data[order(decay_data$t.decay), ]
decay_data <- decay_data[order(decay_data$rep), ]
decay_data <- decay_data[order(as.numeric(decay_data$treatment)), ] 
decay_data <- decay_data[order(decay_data$geneID), ]
ids <- as.character(unique(decay_data$geneID)) # 118 in example set
decay_data[1:10, ]
```


## 4. GENERATE MATRICES OF $\alpha$ AND $\beta$ EQUIVALENCE CONSTRAINT GROUPS
Create objects and set varables used in the modeling based on `decay_data`.
```{r}
# (no changes needed here - just execute the code)
nEquivGrp <- if (length(unique(decay_data$treatment)) == 2) {2} else
  if (length(unique(decay_data$treatment)) == 3) {5} else 
    if (length(unique(decay_data$treatment)) == 4) {15}
genoSet <- 1:(length(unique(decay_data$rep)) * length(unique(decay_data$t.decay)))
nTreat <- length(unique(decay_data$treatment))
nSet <- length(genoSet)*nTreat
groups <- groupings(decay_data)
mods <- data.frame(
  "a" = as.vector(sapply(1:nEquivGrp, function(x) {rep(x, nEquivGrp + 1)})),
  "b" = rep(1:(nEquivGrp + 1), nEquivGrp),
  row.names = paste0("mod", 1:(nEquivGrp*(nEquivGrp+1)))
)
```

Create a colormap of model groups.
```{r}
group_map(decaydata = decay_data, path = paste0(wrdir, "/Model grouping colormap.pdf"), nEquivGrp = nEquivGrp, groups = groups, mods = mods)
```
This file ("Model grouping colormap.pdf") should now be in your working directory. It is a color map reference for understanding model constraints. For example, model 1 has all different colored boxes representing unconstrained $\alpha$s and unconstrained betas, whereas,  the second to last model has only two box colors - one for all $\alpha$s indicating that they all have constrained equivalence and the other indicating that all $\beta$s also have constrained equivalence. Constant decay models (i.e., $\beta$s = 0) are indicated with gray color in the $\beta$ columns. 


## 5. MODEL OPTIMIZATION

Determine the bounds for $\alpha$ and $\beta$ parameters. The bounds on $\alpha$ and $\beta$ are related to distinguishing constant decay, and decaying decay as described in the modeling supplement of [Sorenson et al., 2018](http://www.pnas.org/content/early/2018/01/30/1712312115.short). 

_Care should be taken in determining these bounds given each experimental design._ We have provided functions to aid in selection of the lower bounds of $\alpha$ and $\beta$ (`a_low` and `b_low`, respectively), and upper bound for $\alpha$ (`a_high`). These functions calculate limits based on the time points data were collected. For example, if abundance for an unstable mRNA is measured at the first time point to be 0.02% of the initial (T0) abundance, the information needed to estimate this decay rate was not collected. So, `a_high` caluclates the maximum estimatable decay rate based on the time at which the first decay data point was collected. Similarly, the lower bound for $\beta$ is required to distinguish the constant decay model (`const_decay`) from the decaying decay model (`decaying_decay`). The upper bound for $\beta$ is required to distinguish the no decay model (`const_decay(0,t)`) from decaying decay model. If $\beta$ is too small 1-exp(-$\beta$\*t) ~ -$\beta$\*t and c(t) ~ exp(-$\alpha$\*t); therefore, we can't distinguish between constant decay and decaying decay models. If $\beta$ is too big 1-exp(-$\beta$\*t) ~ 1 and c(t) ~ exp(-$\alpha$/$\beta$), a constant, we can't distinguish between no decay and decaying decay models. Refer to [Sorenson et al. (2018)](http://www.pnas.org/content/early/2018/01/30/1712312115.short) supplemental material for more detail about how we calculated the bounds for $\alpha$ and $\beta$.

```{r}
a_bounds <- c(a_low(max(decay_data$t.decay)),
             a_high(min(unique(decay_data$t.decay)[unique(decay_data$t.decay)>0])))
b_bounds <- c(b_low(max(decay_data$t.decay)), 0.075)
a_bounds;b_bounds
```

Modeling is accomplished using the `mod_optimization` function. The function takes as arguments, the gene identifier, $\alpha$ and $\beta$ bounds, the `decay_data` data frame, a vector of model names to run optimization for (e.g., as `c("mod1", "mod239", ... )`, the equivalence constraint matrix (defined as `groups` above), and a matrix to specify which contstraint groupings to use for $\alpha$ and $\beta$ parameters (defined as `mods` above), respectively, for each model, and a results folder (which will be made if it doesn't already exist) to which the results are written. A number of other arguments are available as well, see help file for details using `?mod_optimization`.  

Efficient model parameter optimization is accomplished using objective functions coded as binary dynamically linked shared libraries. These libraries are compiled from functions coded in the C++ language using functionality of the `TMB` package. The compiled files are dynamically linked library (\*.dll, for Windows) or shared object (\*.so, linux and Mac) files for each of the objective functions. If RNAdecay is installed from source, compiling C++ code will require a compiler be installed separatedly on your system (e.g., Rtools34 or later for Windows,  https://cran.r-project.org/bin/windows/Rtools/; Xcode comand line tools for Mac, http://railsapps.github.io/xcode-command-line-tools.html; R development package for Linux, r-base-dev). If you don't already have one of these installed, install one and restart R. C++ source files are located in the installed 'RNAdecay/src' folder. The compiled (shared library) files should also be written to this same folder upon installation.

Test the modeling on a couple of genes on a handful of models. 
```{r}
mod_optimization("AT2G18150", decay_data, group = groups, mod = mods, a_bounds, b_bounds, c("mod1", "mod2", "mod16", "mod239", "mod240"), file_only = FALSE, path = wrdir) 
mod_optimization("AT4G09680", decay_data, group = groups, mod = mods, a_bounds, b_bounds, c("mod1", "mod2", "mod16", "mod239", "mod240"), file_only = FALSE, path = wrdir) 
```

Next, run all the models on each and every gene. This requires significant computational resources depending on data set size, number of models, and computing speed. The sample data set has 8 time points x 4 replicates each x 4 treatments (= 128 samples) for 118 genes and requires about 10 h processor time. 20000 genes at ~5 min each is ~70 d processor time, so be sure to test a few different models on a few handfuls of genes to feel comfortable running all the modeling. We recommend parallel processing (on multiple processor cores) using `parallel::mclapply` to cut overall time. If this function is unavailable on your machine (e.g., on _Windows_) you can use lapply, but it will take much longer. `mod_optimization()` writes results to file in tab-delimited form, and, optionally, can also return them as a data frame object. 
```{r}
####### To run all genes in parallel use:
# parallel::mclapply(ids, FUN = mod_optimization, 
#                    data = decay_data, group = groups, mod = mods, 
#                    alpha_bounds = a_bounds, beta_bounds = b_bounds,
#                    models = paste0("mod", 1:240), 
#                    path = paste0(wrdir, "/modeling_results"),
#   mc.cores = getOption("mc.cores",  15L), # set the number of compute cores to use here (e.g., 9L = 9 cores, 11L = 11 cores)
#   mc.preschedule = TRUE,
#   mc.set.seed = TRUE,
#   mc.silent = FALSE,
#   mc.cleanup = TRUE,
#   mc.allow.recursive = TRUE)
```

The entire example data set takes ~ 10 h processor time with lapply so for this example we will randomly select one gene ID to run here. 
```{r}
test_ids <- sample(ids, 1) # NOTE: that everytime this line is run it generates a different random sampling, therefore the gene modeled below will be different each time this code is run. To test the exact gene shown in the vignette make a new character vector of the gene id reported below and pass it to the gene argument of mod_optimization using lapply instead of passing 'test_ids' as we do here.  
test_ids

a <- proc.time()[3]
models <- lapply(X = test_ids, # alternatively use `ids` here to complete the entire sample data set, but be prepared to wait ~ 10 h. These gene IDs will get passed one at time to the "gene" argument of mod_optimization() and return a list of the results data frame.
                FUN = mod_optimization,
                data = decay_data, 
                group = groups, 
                mod = mods,
                alpha_bounds = a_bounds, 
                beta_bounds = b_bounds,
                models = rownames(mods), 
                path = paste0(wrdir, "/modeling_results"),
                file_only = FALSE) 
names(models) <- test_ids
b <- proc.time()[3]
(b-a)/60/length(test_ids) # gives you average min per gene 
```

For each gene, read in the results data frame written by `mod_optimization` as an element of a single list object.
```{r}
models <- lapply( paste0( wrdir, "/modeling_results/", test_ids, "_results.txt"), read.delim, header = TRUE )   
names(models) <- test_ids
```


## 6. MODEL SELECTION

The AICc statistic is used to compare model performance. Better models have lower AICc values. Select a single model for each gene based on the lowest AICc statistic. We will continue here with the `RNAdecay::models` data included in the package which uses all 118 genes from the sample data set `RNAdecay::decay_data`.
```{r}
models <- RNAdecay::models # built-in example data, comment this line out to continue with your own modelling output
results <- t(sapply(models, function(x) x[x[, "AICc"] == min(x[, "AICc"]), ]))
results <- as.data.frame(results)
results[, 1:2] <- sapply(as.data.frame(results[, 1:2]), function(x) as.character(unlist(x)))
results[, -c(1,2)] <- sapply(results[, -c(1,2)], unlist)
write.table(results, file = paste0(wrdir,"/best model results.txt"), sep = "\t")
results <- read.delim(paste0(wrdir,"/best model results.txt"))
```

Generate some graphics to visualize results.
```{r}
library(ggplot2)
pdf(paste0(wrdir,"/distributions of stats.pdf"))
p <- ggplot(results)
print(p+geom_histogram(aes(x = sigma2), bins = 300)+
        coord_cartesian(xlim = c(0,0.5))+
        geom_vline(xintercept = 0.0625,color = "red2")+
        plain_theme())
print(p+stat_bin(aes(x = sigma2), breaks = c(seq(0,0.25,0.25/50),1), geom = "bar")+coord_cartesian(xlim = c(0,0.25)))
print(p+stat_ecdf(aes(sigma2), geom = "line")+
        coord_cartesian(xlim = c(0,0.5)))
print(p+stat_ecdf(aes(sqrt(sigma2)), geom = "line")+
        coord_cartesian(xlim = c(0,sqrt(0.5))))
print(p+geom_histogram(aes(x = range.LL), bins = 60))
print(p+geom_histogram(aes(x = nUnique.LL), bins = 60))
dev.off()

pdf(paste0(wrdir,"/lowest AICc model counts.pdf"), height = 8, width = 32)
p <- ggplot(data = data.frame(
  model = as.integer(gsub("mod","",names(table(results$mod)))),
  counts = as.integer(table(results$mod))))+
  geom_bar(aes(x = model, y = counts), stat = "identity")+
  scale_x_continuous(limits = c(0,nrow(mods)), breaks = seq(0,nrow(mods),5))+
  ggtitle("model distribution of absolute lowest AICs")
print(p+plain_theme(25))
dev.off()
```

Because two models are considered to perform differently if their AICc value difference is >2, any models that have AICc values within two of the model with the lowest AICc are considered to have performed similarly. The number of models performing similarly to the best model are plotted as a histogram here. The number of different alpha groupings represented in these models is also plotted.   
```{r}
min_mods <- sapply(models, function(x) which (x[, "AICc"] < (2+min(x[, "AICc"])))) 
min_alpha_mods <- lapply(min_mods, function(x) unique(mods[x, "a"]))

pdf(paste0(wrdir,"/number of models that performed similar to the one selected.pdf"))
barplot(height = table(sapply(min_mods, length)), xlab = "No. models in the lowest AICc group (not more than 2 different from lowest)",
        ylab = "No. genes")
barplot(height = table(sapply(min_alpha_mods, length)), xlab = "No. alpha groups in the lowest AICc group (not more than 2 different from lowest)",
        ylab = "No. genes")
dev.off()
```

Build the modeling results data frame. Group genes with similar $\alpha$ groupings and then group genes with similar $\beta$ groupings.
```{r}
results <- read.delim(paste0(wrdir,"/best model results.txt"))
results$alpha_grp <- mods[as.character(results$mod), "a"]
results$beta_grp <- mods[as.character(results$mod), "b"]
results$mod <- as.numeric(gsub("mod", "", as.character(results$mod)))

results$alphaPattern <- sapply(rownames(results), function(x) {
  paste0(gsub("alpha_", "", colnames(results)[3:(2+nTreat)][order(unlist(round(results[x, 3:(2+nTreat)], 4)))]), collapse = "<=")
  })
results$alphaPattern <- paste0(results$alpha_grp, "_", results$alphaPattern)
results$betaPattern <- sapply(rownames(results), function(x){
  paste0(gsub("beta_", "", colnames(results)[(3+nTreat):(2+2*nTreat)][order(unlist(round(results[x, (3+nTreat):(2+2*nTreat)], 4)))]), collapse = "<=")
  })
results$betaPattern <- paste0(results$beta_grp, "_", results$betaPattern)

results <- results[order(rownames(results)), ]
results <- results[order(results$beta_grp), ]
results <- results[order(results$alphaPattern), ]
results <- results[order(results$alpha_grp), ]

results$alphaPattern <- factor(results$alphaPattern, levels = as.character(unique(results$alphaPattern)))

results <- data.frame(results[, 3:(2*nTreat+3), 2], results[, c("AICc", "alpha_grp", "beta_grp", "alphaPattern", "betaPattern")])
results$nEqMods <- sapply(min_mods[rownames(results)], length)
results$nEqAgp <- sapply(min_alpha_mods[rownames(results)], length)

# Customize: add columns of relative alphas and betas as desired, e.g.:
results$rA_sov.WT    <- results$alpha_sov      / results$alpha_WT
results$rA_vcs.WT    <- results$alpha_vcs      / results$alpha_WT
results$rA_vcssov.WT <- results$alpha_vcs.sov  / results$alpha_WT

write.table(results, paste0(wrdir,"/alphas+betas+mods+grps+patterns+relABs.txt"), sep = "\t")

results <- read.delim(paste0(wrdir,"/alphas+betas+mods+grps+patterns+relABs.txt"), header = TRUE, colClasses =  c(NA, rep("numeric", 2+2*nTreat), rep("integer", 2), rep("character", 2), rep("integer", 2), rep("numeric", 3)))
# results$alpha_subgroup <- factor(results$alpha_subgroup, levels = unique(results$alpha_subgroup))
results$alphaPattern <- factor(results$alphaPattern, levels = unique(results$alphaPattern))
results$betaPattern <- factor(results$betaPattern, levels = unique(results$betaPattern))
results[1:3, ]
```

All files were written to the `wrdir`. The modeling results are now ready to be visualized.



# III. DATA VISULIZATION


## 7. LOAD DECAY DATA AND MODELING RESULTS (AND, OPTIONALLY, GENE DESCRIPTIONS)

Load normalized read data and reorder factor levels of "treatment" as you would like them to appear in the plot key and in the order you want them to plot.
```{r}
library(RNAdecay)
decay_data <- RNAdecay::decay_data # built-in package example data; comment this line out to use your own decay_data
decay_data$treatment <- factor(decay_data$treatment, levels = c("WT", "sov", "vcs", "vs")) # you must type them identically in the new order  
levels(decay_data$treatment)[4] <- "vcs.sov"
decay_data[1:4,]
```

Load modeling results.
```{r}
results <- RNAdecay::results # built-in package example data; comment this line out to use your own results
# For example:
# results <- read.delim(paste0(tempdir(),"/DecayAnalysis/Example analysis results 2/alphas+betas+mods+grps+patterns+relABs.txt"), header = TRUE, colClasses =  c(NA, rep("numeric", 9), rep("integer", 3), rep("character", 3), rep("numeric", 6)))
# results$alpha_subgroup <- factor(results$alpha_subgroup, levels = unique(results$alpha_subgroup))
results[1:3, ]
```

Optionally, for printing gene descriptions directly on the plot, load a gene description file and then generate a character vector of descriptions with elements named with geneIDs 
(e.g., for _Arabidopsis thaliana_, download and unzip   https://www.arabidopsis.org/download_files/Genes/TAIR10_genome_release/gene_description_20131231.txt.gz, and place it in your working directory).
```{r}
# gene_desc <- read.delim("gene_description_20140101.txt"), quote = NULL, comment = '', header = FALSE)
# gene_desc[, 1] <- substr(gene_desc[, 1], 1, 9)
# gene_desc <- data.frame(gene_desc[!duplicated(gene_desc[, 1]), ], row.names = 1)
# colnames(gene_desc) <- c("type", "short description", "long description", "computational description")
# descriptions <- gene_desc[, "long description"]
# names(descriptions) <- rownames(gene_desc)

descriptions <- c(
  "Encodes a ubiquitin E3 ligase with RING and SPX domains that is involved in mediating immune responses and mediates degradation of PHT1s at plasma membranes.  Targeted by MIR827. Ubiquitinates PHT1;3, PHT1;2, PHT1;1/AtPT1 and PHT1;4/AtPT2.",
  "",
  "Related to Cys2/His2-type zinc-finger proteins found in higher plants. Compensated for a subset of calcineurin deficiency in yeast. Salt tolerance produced by ZAT10 appeared to be partially dependent on ENA1/PMR2, a P-type ATPase required for Li+ and Na+ efflux in yeast. The protein is localized to the nucleus, acts as a transcriptional repressor and is responsive to chitin oligomers. Also involved in response to photooxidative stress.",
  "Encodes a stress enhanced protein that localizes to the thylakoid membrane and whose mRNA is upregulated in response to high light intensity.  It may be involved in chlorophyll binding."
  )
names(descriptions) <- c("AT1G02860","AT5G54730", "AT1G27730", "AT4G34190")
```

## 8. PRINT DECAY PLOTS TO PDF

Plot a single gene using `decay_plot`. If you do not have a gene desription file, do not include "Desc" in the `what` argument (see `?decay_plot`). 
```{r, fig.show = 'hold'}
p <- decay_plot(geneID = "AT1G02860", 
              xlim = c(0, 350), 
              ylim = c(0, 1.25), 
              xticks = 1:5*100, 
              yticks = 0:5/4,
              alphaSZ  =  12, 
              what  =  c("meanSE", "reps", "models"),
              treatments  =  c("WT", "vcs.sov"), 
              colors = c("darkblue", "red3"),
              DATA = decay_data, 
              mod.results = results, 
              gdesc = descriptions)
# print(p+plain_theme(8, 1, leg.pos = c(0.7, 0.8))) #this will print the plot to your current graphics device (dev.cur() tells you what that is), if you do not have a graphics device open (e.g., "null device") initiate one (e.g., use quartz(),pdf(), or windows(); dev.off() closes the device and writes the file).
```

Plot one or multiple genes; write them to a pdf file. 
```{r}
# make new directory for results
wrdir <- paste0(tempdir(),"/DecayAnalysis/Example analysis results 3")
if(!file.exists(wrdir)) dir.create(wrdir, recursive = TRUE)

pdf(paste0(wrdir, "/decay plot example.pdf"), width = 10, height = 8)
p <- decay_plot(geneID = "AT1G02860", 
              xlim = c(0, 500), 
              ylim = c(0, 1.25), 
              xticks = 1:5*100, 
              yticks = 0:5/4,
              alphaSZ = 10, 
              what = c("Desc","meanSE", "reps", "models","alphas&betas"),
              treatments = c("WT", "vcs.sov"),
              colors = c("darkblue", "red3"),
              DATA = decay_data,
              mod.results = results,
              gdesc = descriptions)
print(p+plain_theme(32, 1, leg.pos = 'right'))
dev.off()

# plot multiple genes
ids <- c("AT5G54730", "AT1G27730", "AT4G34190")
# or e.g.,
# ids <- rownames(results[results$alpha_WT > 0.1, ]) 

pdf(paste0(wrdir, "/multiple RNA decay plots.pdf"), width = 10, height = 7)
for(i in ids){
  p <- decay_plot(i, 
                what = c("meanSE", "models", "Desc"),
                mod.results = results, 
                gdesc = descriptions,
                treatments = c("WT","sov","vcs","vcs.sov"),
                DATA = decay_data)
  print(p+plain_theme(13.8, 1, leg.pos = 'right'))
  cat(i, " plotted.\n")
}
dev.off()
```


Plot one or multiple genes using hl_plot; write them to a pdf file. 
```{r}

pdf(paste0(wrdir, "/halflife plot example.pdf"), width = 10, height = 8)

p <- hl_plot(geneID = "AT1G02860",
             gene_symbol = "SYG1",
             df_decay_rates = RNAdecay::results,
             hl_dist_treatment = "WT",
             hl_treatment = c("WT","sov","vcs","vcs.sov"),
             arrow_lab_loc = "key"
             )
  
print(p)
dev.off()

# plot multiple genes
ids <- c("AT5G54730", "AT1G27730", "AT4G34190")
# or e.g.,
# ids <- rownames(results[results$alpha_WT > 0.1, ]) 

pdf(paste0(wrdir, "/multiple halflife plots.pdf"), width = 4, height = 4)
for(i in ids) {
  
p <- hl_plot(geneID = i,
             df_decay_rates = RNAdecay::results,
             hl_dist_treatment = "WT",
             hl_treatment = c("WT","sov","vcs","vcs.sov"),
             arrow_lab_loc = "plot"
             )
  print(p+plain_theme(8, 1,leg.pos = 'right'))
  cat(i, " plotted.\n")

  }
dev.off()
```


## 9. Session Information of Most Recent Update

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sse.R
\name{sse_null_decaying_decay}
\alias{sse_null_decaying_decay}
\alias{sse_null_const_decay}
\title{sum of the squared errors for null models}
\usage{
sse_null_decaying_decay(a, b, m, t)

sse_null_const_decay(a, m, t)
}
\arguments{
\item{a}{alpha value}

\item{b}{beta value}

\item{m}{mRNA abundance values}

\item{t}{time points of \code{m}}
}
\value{
Returns the sum of the squared errors
}
\description{
For a model that uses a constant decay rate or a decaying decay rate, calculates the sum of the squared errors (differences between the supplied data points and the modeled values based on alpha and/or beta values). For these models all treatments are assumed to have the same \code{a} (alpha) and/or \code{b} (beta).
}
\examples{
sse_null_decaying_decay(a=0.05, b = 0.001,
           m = c(1,1,1,0.99,0.5,0.5,0.5,0.49,0.25,0.25,0.25,0.24,0.12,0.125,0.125,0.126),
           t = rep(c(0,10,20,30),each = 4))
sse_null_const_decay(a=0.05,
           m = c(1,1,1,0.99,0.5,0.5,0.5,0.49,0.25,0.25,0.25,0.24,0.12,0.125,0.125,0.126),
           t = rep(c(0,10,20,30),each = 4))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aic.R
\name{aic}
\alias{aic}
\alias{aicc}
\title{Akaike information criterion (with correction)}
\usage{
aic(maxlLik, p)

aicc(maxlLik, p, n)
}
\arguments{
\item{maxlLik}{maximum log likelihood value identified upon model convergence}

\item{p}{number of parameters in the model}

\item{n}{is the total number of observations of a single gene (e.g., 8 time
points X 4 replicates X 4 treatments/genotypes = 128)}
}
\value{
returns the AIC or AICc values
}
\description{
Calculates AIC or AICc.
}
\examples{
aicc(100,5,15)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/group_map.R
\name{group_map}
\alias{group_map}
\title{model color map}
\usage{
group_map(decaydata, path, nEquivGrp = nEquivGrp, groups = groups, mods = mods)
}
\arguments{
\item{decaydata}{5 column data.frame with colnames "geneID","treatment","t.decay","rep","value"}

\item{path}{write path and file name, must end in ".pdf"}

\item{nEquivGrp}{number of equivalence groups based on number of treatments}

\item{groups}{equivalence group matrix}

\item{mods}{alpha beta equivalence group usage index (matrix)}
}
\value{
creates a model colormap and writes it to a pdf file named \code{path}
}
\description{
group_map makes a color map of alpha and beta equivalence groups by model. Similar colors in a row indicate constrained parameter equivalence between treatements. Gray indicates values of 0.
}
\examples{
group_map(decaydata=data.frame(geneID=paste0("gene",1:4),
                    treatment=as.factor(rep(paste0("treat",1:2),2)),
                    t.decay=0:3,
                    rep=rep("rep1"),
                    value=c(1,0.5,0.25,0.12)),
         path=paste0(tempdir(),"/parameter equivalence colormap.pdf"),
         nEquivGrp = 2,
         groups = t(matrix(c(1,2,1,1,NA,NA),nrow=2,
                    dimnames=list(c("treat1","treat2"),c("grp1","grp2","grp3")))),
         mods = t(matrix(c(1,1,1,2,1,3,2,1,2,2,2,3),nrow=2,
                         dimnames=list(c("a","b"),paste0("mod",1:6)))))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Example_RPMs.R
\docType{data}
\name{RPMs}
\alias{RPMs}
\title{RNA abundance reads per million over RNA decay timecourse}
\format{
a data frame with 118 rows and 128 columns; data are all RNA abundance values presented as reads per million. Column names indicate genotype, time point, and replicate number separated by underscores.
}
\source{
Sorenson et al. (2017) Submitted; \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE86361}
}
\usage{
RPMs
}
\description{
A dataset of RNA abundance of 118 genes in four Arabidopsis thaliana genotypes (WT, sov, vcs, vcs sov). Four biological replicates were collectred 0, 7.5, 15, 30, 60, 120, 240, 480 min after blocking transcription. RNA was extracted, subjected to ribodepletion, and sequenced by RNA-seq (Illumina 50 nt single end reads).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plain_theme.R
\name{plain_theme}
\alias{plain_theme}
\title{a custom ggplot2 theme}
\usage{
plain_theme(bigFont = 30, smFont = 0.85, x.ang = 0, leg.pos = c(0.85, 0.85))
}
\arguments{
\item{bigFont}{larger font size of axis labels in points (used for plot title, axis titles, facet titles)}

\item{smFont}{fractional mulitiplier of \code{bigFont} (used for axis text)}

\item{x.ang}{x-axis label angle}

\item{leg.pos}{legend position on plot as relative coordinates c(x,y) (i.e., range is [0,1]) or 'right', 'left', 'above', 'below'}
}
\value{
returns a ggplot2 theme of class "theme" "gg"
}
\description{
A custom ggplot2 theme generating function for ggplot2 plots; can be further manipulated using standard ggplot2 syntax.
}
\examples{
plain_theme(10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hl_plot.R
\name{hl_plot}
\alias{hl_plot}
\title{hl_plot() function}
\usage{
hl_plot(
  geneID,
  gene_symbol = "",
  df_decay_rates,
  hl_dist_treatment,
  hl_treatment,
  arrow_colors = NA,
  arrow_lab_loc = c("key"),
  x_limits = log(2)/c(0.25, 0.00045),
  x_breaks = c(5, 1:12 * 10, 180, 240, 300, 360, 420, 480, 720, 1080, 1440),
  x_tick_labels = c("5", "10", "", "30", "", "", "60", "", "", "", "", "", "2h", "",
    "4h", "", "", "", "8h", "12h", "", "24h")
)
}
\arguments{
\item{geneID}{single gene ID from data set (e.g. "AT3G14100") for which to plot data/model}

\item{gene_symbol}{(optional) pasted to gene ID in plot label (e.g., "AT3G14100/UBP1C)}

\item{df_decay_rates}{data.frame of modeling results with decay rate columns labeled as alpha_<treatment>}

\item{hl_dist_treatment}{name of the treatment for which the background distribution will be plotted}

\item{hl_treatment}{names of the treatments for which arrows indicating half-life will be plotted}

\item{arrow_colors}{(optional) character vector of R colors; named with corresponding treatments}

\item{arrow_lab_loc}{label arrows on plot ("plot") or in a key ("key")}

\item{x_limits}{x-axis (half-life) limits in min; default is log(2)/c(0.25,4.5e-4)}

\item{x_breaks}{x-axis (half-life) breaks/tick marks in min defaults to c(5,1:12*10,180,240,300,360,420,480,720,1080,1440)}

\item{x_tick_labels}{x-axis (half-life) break labels, defaults to c("5","10","","30","","","60","","","","","","2h","","4h","","","","8h","12h","","24h")+}
}
\value{
returns a ggplot to be used with print; could also be modified using the syntax of ggplot2 e.g.'+geom_XXXX(...)'
}
\description{
Plots RNA half-life distribution with select half-lives of select RNAs as large arrows colored by treatment using the ggplot2 package.
}
\examples{

p <- hl_plot(
geneID = rownames(RNAdecay::results)[4],
df_decay_rates = RNAdecay::results,
hl_treatment = c("WT","sov","vcs","vcs.sov"),
hl_dist_treatment = "WT",
arrow_colors = c(WT = "#88CCEE", sov = "#CC6677", vcs = "#117733",vcs.sov = "#882255"),
arrow_lab_loc = "key",
gene_symbol = ""
)

print(p)

p <- hl_plot(
geneID = rownames(RNAdecay::results)[4],
gene_symbol = "",
df_decay_rates = RNAdecay::results,
hl_dist_treatment = "WT",
hl_treatment = c("WT","sov","vcs","vcs.sov"),
arrow_colors = c(WT = "#88CCEE", sov = "#CC6677", vcs = "#117733",vcs.sov = "#882255"),
arrow_lab_loc = "plot"
)

print(p)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constraint_fun_list_maker.R
\name{constraint_fun_list_maker}
\alias{constraint_fun_list_maker}
\title{contraint function list maker}
\usage{
constraint_fun_list_maker(mods, groups)
}
\arguments{
\item{mods}{data.frame specifying alpha and beta group pairs for each model}

\item{groups}{grouping matrix for alphas or betas}
}
\value{
Returns a list of constraint functions to be passed to the
optimization function.
}
\description{
Individual double exponential models are all nested within model number 1 in
which alpha and beta parameters vary independently for each treatment. Models
that assume no difference in parameters between specific treatments manifest
as constraints in the modeling. These constraints are coded as functions
that are passed to the optimization process. Each model has a distinct
constraint function.
}
\examples{
constraint_fun_list_maker(mods = data.frame(a = c(1,1,1,2,2,2), b = c(1,2,3,1,2,3),
                          row.names = paste0('mod',1:6)),
                          groups = data.frame(treat1 = c(1,1,NA), treat2 = c(2,1,NA)))

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Example_results.R
\docType{data}
\name{results}
\alias{results}
\title{Example double exponential decay modeling results}
\format{
a data frame with 18 columns and 118 rows.
 \describe{
    \item{alpha_XXX}{decay rate estimate of genotype XXX, in per time (min^-1)}
    \item{beta_XXX}{decay of decay rate estimate of genotype XXX, in per time (min^-1)}
    \item{sigma2}{variance estimate}
    \item{model}{selected model number}
    \item{alpha_grp}{model alpha grouping number }
    \item{beta_grp}{model beta grouping number }
    \item{alpha_subgroup}{model alpha subgroup number}
    \item{alphaPattern}{model alpha subgroup pattern; i.e. order of genotypes of increaseing decay rate}
    \item{betaPattern}{model beta subgroup pattern; i.e. order of genotypes of increaseing decay of decay rate}
    \item{rA_XXX}{relative alpha value of genotype XXX compared to WT}
    \item{nEqMods}{number of models that were not different than the selected model based on a AICc difference <2}
    \item{nEqAgp}{number of alpha groups represented in nEqMods}
    }
}
\source{
Sorenson et al. (2017) Submitted; \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE86361}
}
\usage{
results
}
\description{
Example results from maximum likelihood modeling of double exponential RNA decay of 118 genes. Results include parameter estimates, selected model, and alpha and beta groupings.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/n_par.R
\name{n_par}
\alias{n_par}
\title{number of Parameters function}
\usage{
n_par(model, mod, group)
}
\arguments{
\item{model}{model name (e.g. 'mod1')}

\item{mod}{two column data.frame with combination of alpha and beta grouping numbers in each model with rownames 'modX'}

\item{group}{matrix of all treatment alpha or beta equivalence groups}
}
\value{
returns the integer value of number of parameters in \code{model}
}
\description{
Calculates number of parameters for a specified model given the model parameter constraints.
}
\examples{
n_par('mod1',data.frame('a'=1:5,'b'=rep(2,5),row.names=paste0('mod',1:5)),
         t(matrix(c(1,2,3,4,1,2,2,2,1,1,2,2,1,1,1,2,1,2,1,2,1,2,2,1),nrow=4)))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/log_lik.R
\name{log_lik}
\alias{log_lik}
\title{log likelihood}
\usage{
log_lik(x, y, n)
}
\arguments{
\item{x}{is the minimum SSE from the slsqp() fitting}

\item{y}{is sigma2 cooresponding to the minimum SSE}

\item{n}{is the total number of observations of a single gene (e.g., 8 time points X 4 replicates X 4 treatments/genotypes = 128)}
}
\value{
returns Log Likelihood
}
\description{
Calculates the log likelihood value from the sum of the squared errors, sigma2, and the total number of data points.
}
\examples{
log_lik(1,1/128,128)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cols.R
\name{cols}
\alias{cols}
\title{Indexes column names of a data.frame matching multiple patterns (i.e., multigrep)}
\usage{
cols(patterns, df, w = NA, x = NA, y = NA, z = NA)
}
\arguments{
\item{patterns}{character vector or vector of regular expressions passed to grep pattern argument}

\item{df}{a dataframe with column names to index}

\item{w, x, y, z}{(for backwards compatibility) separate arguments for patterns, if used patterns argument will be ignored}
}
\value{
returns a vector of integer indices of the column names of \code{df} that match to all of \code{patterns}
}
\description{
Identifies dataframe column names that have all of the pattern arguments .
}
\details{
Be aware that column data labels that are part of another data label are not advisable (e.g. mut1, mut2, mut1.mut2; cols(df,'mut1') will return indices for both 'mut1' and 'mut1.mut2' labeled columns
}
\examples{
cols(df=data.frame(xyz=1:5,zay=6:10,ybz=11:15,tuv=16:20),patterns = c('y','z')) ## returns 1 2 3
cols(df=data.frame(xyz=1:5,zay=6:10,ybz=11:15,tuv=16:20), w = 'y', x = 'z') ## returns 1 2 3

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comb_cv.R
\name{comb_cv}
\alias{comb_cv}
\title{combined adjusted coefficient of variation}
\usage{
comb_cv(X, eps = 1e-04)
}
\arguments{
\item{X}{data.frame or matrix of numeric data}

\item{eps}{small value to add to the mean to avoid dividing by 0; defaults to 1e-4}
}
\value{
returns the sum of the coefficients of variation for all columns of \code{X}
}
\description{
Calculates the sum of the column standard devation divided by the sum of the column mean and a small value to avoid dividing by 0 (\code{eps})
}
\examples{
comb_cv( data.frame( test1=rep(0,5), test2=c(0.2,0.3,0.35,0.27,0.21) ) )
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_optimization.R
\name{mod_optimization}
\alias{mod_optimization}
\title{model optimization for fitting exponential decay models to normalized data}
\usage{
mod_optimization(
  gene,
  data,
  alpha_bounds,
  beta_bounds,
  models,
  group,
  mod,
  file_only = TRUE,
  path = "modeling_results"
)
}
\arguments{
\item{gene}{geneID from \code{data} to be modeled}

\item{data}{decay data data.frame with columns named 'geneID', 'treatment',
't.decay', 'rep', 'value.'}

\item{alpha_bounds}{vector of length 2 with lower and upper bounds for
alpha}

\item{beta_bounds}{vector of length 2 with lower and upper bounds for beta}

\item{models}{vector spceifying which models to run optimization on (e.g.,
c('mod1', 'mod239'))}

\item{group}{grouping matrix for alphas or betas}

\item{mod}{data.frame specifying alpha and beta group pairs for each model}

\item{file_only}{logical; should output only be written to file (TRUE) or
also return a data.frame of the results (FALSE)}

\item{path}{specify folder for output to be written}
}
\value{
returns (if \code{file_only = FALSE}) and writes to \code{path} a
  data frame of model optimization results for \code{models} one row for
  each for \code{gene} using values for it found in \code{data}, the columns
  of the data frame are:
geneID, mod (model), model estimates [alpha_treatment1, ...,
  alpha_treatmentn, beta_treatment1, ..., beta_treatmentn, sigma2],
 logLik (maximum log likelihood), nPar (number of parameters in the model),
 nStarts (number of parameter starting value sets (of 50) that converged on
 a maximum likelihood peak), J (number of parameter starting value sets that
 converged on the highest - within 1e-4 - maximum likelihood of all
 parameter starting value sets), range.LL (range of maximum likelihoods
 values reached by algorithm convergence from all parameter starting value
 sets), nUnique.LL (number of unique maximum likelihoods values reached by
 algorithm convergence from all parameter starting value sets), C.alpha (sum
 of all coefficients of variation for each column of alpha estimates),
 C.beta (sum of all coefficients of variation for each column of beta
 estimates), C.tot (C.alpha+C.beta), AICc (calculated from the single
 highest maximum likelihood of all parameter starting value sets), AICc_est
 (calculated from the log likelihood value computed by using the mean of
 each parameter from all optimizations that converged on the highest
 maximum likelihood of all starting parameter value sets.)
}
\description{
The mod_optimization function finds the estimates of model parameters by
  maximum likelihood, for a single gene on a specified list of models, and
  saves a tab delimited text file of the results named,'
  [geneID]_results.txt'.
The function does the following for each gene:
(1) it calculates log likelihood for each point in a 2 dimensional grid of
  evenly spaced alpha and beta values within the alpha and beta bounds
  specified using the null model (in which all treatment alphas are
  equivalent and all betas are equivalent).
(2) it calculates log likelihood for each point in a 1 dimensional range of
  evenly spaced alpha values within the alpha bounds using the single
  exponential null model (in which all treatment alphas are equivalent).
(3) For each of the grid points with the highest log likelihood from steps
  (1) and (2) 25 starting parameter value sets that are normally
  distributed around these points are generated.
(4) Parameter values are optimized for maximum likelihood using each of
  these 50 starting parameter sets using pre-compiled C++ functions loaded
 from dynamically linked libraries stored in the package on all models
 specified in the models argument.
(5) evaluates parameter estimates of all 50 optimizations based on the
  reported maximum liklihood upon convergence. Only parameter estimates
  that converged on the same and highest maximum likelihood are returned.
(6) returns the optimized parameter estimates, with model selection
  statistics.
}
\examples{

mod_optimization(gene = 'Gene_BooFu',
                data = data.frame(geneID=rep('Gene_BooFu',30),
                            treatment=c(rep('WT',15),rep('mut',15)),
                            t.decay=rep(c(0,7.5,15,30,60),6),
                            rep=rep(paste0('rep',c(rep(1,5),rep(2,5),rep(3,5))),2),
                            value= c(0.9173587, 0.4798672, 0.3327807, 0.1990708, 0.1656554,
                                     0.9407511, 0.7062988, 0.3450886, 0.3176824, 0.2749946,
                                     1.1026497, 0.6156978, 0.4563346, 0.2865779, 0.1680075,
                                     0.8679866, 0.6798788, 0.2683555, 0.5120951, 0.2593122,
                                     1.1348219, 0.8535835, 0.6423996, 0.5308946, 0.4592902,
                                     1.1104068, 0.5966838, 0.3949790, 0.3742632, 0.2613560)),
                alpha_bounds = c(1e-4,0.75),
                beta_bounds = c(1e-3,0.075),
                models = 'mod1',
                group = t(matrix(c(1,2,1,1,NA,NA),nrow=2,
                          dimnames=list(c('treat1','treat2'),c('mod1','mod2','mod3')))),
                mod = as.data.frame(t(matrix(c(1,1,1,2,1,3,2,1,2,2,2,3),nrow=2,
                        dimnames=list(c('a','b'),paste0('mod',1:6))))),
                file_only = FALSE,
                path = paste0(tempdir(),"/modeling results"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decay_plot.R
\name{decay_plot}
\alias{decay_plot}
\title{decay_plot() function}
\usage{
decay_plot(
  geneID,
  xlim = c(0, 500),
  ylim = c(0, 1.25),
  xticks = NA,
  yticks = 0:5/4,
  alphaSZ = 8,
  what = c("Desc", "models", "reps", "meanSE", "alphas&betas"),
  DATA,
  treatments = NA,
  colors = NA,
  mod.results = NA,
  gdesc = NA,
  desc.width = 55
)
}
\arguments{
\item{geneID}{single gene ID from data set (e.g. "AT1G00100") for which to plot data/model}

\item{xlim, ylim}{vector of length 2 defineing the limits of the plot (zooms in on data)}

\item{xticks, yticks}{vectors specifyng tick marks for the x and y axes}

\item{alphaSZ}{text size of alpha and beta parameter labels if plotted}

\item{what}{character vector specifying what to plot; any or all (default) of "Desc","models","reps","meanSE","alphas&betas"
"Desc"         - plots gene descriptions behind data
"models"       - plots the selected fit model
"reps"         - plots individual replicate data as distinct shapes
"meanSE        - plots the replicate means and standard errors
"alphas&betas" - plots the values of the alphas and betas for each model below the model at the greatest x position}

\item{DATA}{(required) normalized abundance decay data with column names: "geneID", "treatment","t.decay", "rep","value"}

\item{treatments}{what treatments/genotypes to plot from the supplied data}

\item{colors}{vector of R recognized colors (e.g. "red","darkblue")}

\item{mod.results}{(optional; required for plotting models) data.frame of the model results as output from the modeling (e.g. "alphas+betas+mods+grps+patterns+relABs.txt")}

\item{gdesc}{(optional; required for plotting gene descriptions) gene descriptions (geneID-named vector of gene descriptions geneID must match those of data)}

\item{desc.width}{width of gene descriptions (in number of characters) before word wrap}
}
\value{
returns a ggplot to be used with print; could also be modified using the syntax of ggplot2 e.g.'+geom_XXXX(...)'
}
\description{
Plots RNA decay data and/or decay models using the ggplot2 package.
}
\examples{
p<-decay_plot("Gene_BooFu",
          mod.results = data.frame(alpha_WT = 0.0830195, beta_WT = 0.04998945,
                                   model = 1, alpha_grp = 1, beta_grp = 1, alpha_subgroup = 1.1,
                                   row.names = "Gene_BooFu"),
          what = c("meanSE","alphas&betas","models"),
          treatments = "WT",
          colors = "black",
          DATA = data.frame(geneID=rep("Gene_BooFu",15),
                            treatment=rep("WT",15),
                            t.decay=rep(c(0,7.5,15,30,60),3),
                            rep=paste0("rep",c(rep(1,5),rep(2,5),rep(3,5))),
                            value= c(0.9173587, 0.4798672, 0.3327807, 0.1990708, 0.1656554,
                                     0.9407511, 0.7062988, 0.3450886, 0.3176824, 0.2749946,
                                     1.1026497, 0.6156978, 0.4563346, 0.2865779, 0.1680075)),
          xlim = c(0, 65),
          alphaSZ = 10)
print(p)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupings.R
\name{groupings}
\alias{groupings}
\title{Combinatorial groups matrix generator}
\usage{
groupings(decaydata)
}
\arguments{
\item{decaydata}{a data.frame with column names:
'geneID','treatment','t.decay','rep','value' with classes
\code{factor}, \code{factor}, \code{numeric}, \code{factor}, \code{numeric}}
}
\value{
returns a matrix of equivalence group indicies based on the number of levels in the 'treatment' column (max of 4).
}
\description{
Generates a combinatorial grouping matrix based on the \code{decaydata} data.frame.
}
\details{
The resulting matrix of indices is used to constrain treatment alphas or treatment betas in combination. For example, in one model, treatment alphas might be allowed to vary independently (gp1), but the beta models might be constrained to be equal for some treatments indicated by haveing the same index number (other gp).
}
\examples{
groupings(data.frame(geneID=paste0('gene',1:4),treatment=as.factor(paste0('treat',1:4)),
                     t.decay=0:3,rep=rep('rep1'),value=c(1,0.5,0.25,0.12)))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_var.R
\name{fit_var}
\alias{fit_var}
\title{sigma^2 estimation}
\usage{
fit_var(sse, n)
}
\arguments{
\item{sse}{is the minimum SSE (sum of the squared errors) from the slsqp() fitting}

\item{n}{is the total number of observations of a single gene (e.g., 8 time points X 4 replicates X 4 treatments/genotypes = 128)}
}
\value{
returns the sigma squared estimate
}
\description{
Calculates the variance (sigma^2) estimate from the sum of the squared errors from the fit model.
}
\examples{
fit_var(1,128)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bounds.R
\name{a_high}
\alias{a_high}
\alias{a_low}
\alias{b_low}
\title{calculates bounds for modeled parameters}
\usage{
a_high(t_min)

a_low(t_max)

b_low(t_max)
}
\arguments{
\item{t_min}{time of first experiemtal time point after inhibition of
transcription (not T0)}

\item{t_max}{time of last experimental time point}
}
\value{
returns the lowest/highest parameter values to be used as bounds on
  modeled parameters
}
\description{
Calculates maximum and minimum bounds for parameter alpha based on experimental
  time points (t_0, t_1, t_2, t_3, ..., t_max). If RNA level is too low at t_1,
  then the decay has happened before our observations began - there is an upper
  bound to the decay rate we can detect (a_high).
If RNA level is too high at t_max, then relatively little decay has happened and
  we can not distinguish the decay rate and the decay of the decay rate  - there
  is a lower bound to the base decay rate of the decaying decay model (a_low).
}
\details{
Similarly, limits on beta are required to prevent precude ranges in which the
  decay rate and decaing decay are indistinguishable. See vignette
  "RNAdecay_workflow" for more information.
}
\examples{
a_high(7.5)
a_low(480)
b_low(480)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decay_functions.R
\name{const_decay}
\alias{const_decay}
\alias{decaying_decay}
\title{exponential decay functions}
\usage{
const_decay(t, a)

decaying_decay(t, par)
}
\arguments{
\item{t}{time (in minutes)}

\item{a}{alpha (in per time, thus in per minute when time is in minutes)}

\item{par}{vector of length 2 containing alpha (par[1]) and beta (par[2]) values; alpha=initial decay rate, beta=decay of decay rate (both in per time, thus in per minute when time is in minutes)}
}
\value{
returns abundance after time \code{t}  at alpha initial decay rate and beta decay of decay rate relative to an initial abundance of 1
}
\description{
Constant decay rate function (const_decay(), case when betas=0) e^-a*t; decaying decay rate function (decaying_decay())  e^(-(a/b)*(1-e^(-b*t))). Functions are normalized so at t=0 the function is 1.
}
\examples{
const_decay(10,log(2)/10) ## returns 0.5
decaying_decay(10,c(log(2)/10,0.01)) ##returns 0.5170495
}
